@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -19,7 +19,9 @@
 #ifndef gscolor_INCLUDED
 #  define gscolor_INCLUDED
 
+#include "stdpre.h"
 #include "gxtmap.h"
+#include "gsgstate.h"
 
 /* Color and gray interface */
 int gs_setgray(gs_gstate *, double);
