@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -313,7 +313,7 @@ gscms_transform_color_buffer(gx_device *dev, gsicc_link_t *icclink,
     int planar,numbytes, big_endian, hasalpha, k;
     unsigned char *inputpos, *outputpos;
 #if DUMP_CMS_BUFFER
-    FILE *fid_in, *fid_out;
+    gp_file *fid_in, *fid_out;
 #endif
     /* Although little CMS does  make assumptions about data types in its
        transformations you can change it after the fact.  */
@@ -445,8 +445,8 @@ gscms_transform_color_buffer(gx_device *dev, gsicc_link_t *icclink,
         }
     }
 #if DUMP_CMS_BUFFER
-    fid_in = gp_fopen("CM_Input.raw","ab");
-    fid_out = gp_fopen("CM_Output.raw","ab");
+    fid_in = gp_fopen(dev->memory,"CM_Input.raw","ab");
+    fid_out = gp_fopen(dev->memory,"CM_Output.raw","ab");
     fwrite((unsigned char*) inputbuffer,sizeof(unsigned char),
                             input_buff_desc->row_stride,fid_in);
     fwrite((unsigned char*) outputbuffer,sizeof(unsigned char),
