@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -25,6 +25,10 @@
 #include "gsicc_cms.h"
 #include "gxdevice.h"
 
+#ifdef WITH_CAL
+#include "cal.h"
+#endif
+
 #ifndef MEMENTO_SQUEEZE_BUILD
 #define USE_LCMS2_LOCKING
 #endif
@@ -169,7 +173,7 @@ static cmsPluginMutex gs_cms_mutexhandler =
 {
     {
         cmsPluginMagicNumber,
-        2060,
+        LCMS_VERSION,
         cmsPluginMutexSig,
         NULL
     },
@@ -313,8 +317,8 @@ gscms_get_profile_handle_mem(unsigned char *buffer, unsigned int input_size,
 {
     cmsContext ctx = gs_lib_ctx_get_cms_context(mem);
 
-    cmsSetLogErrorHandlerTHR(ctx, gscms_error);
-    return cmsOpenProfileFromMemTHR(ctx,buffer,input_size);
+    cmsSetLogErrorHandler(ctx, gscms_error);
+    return cmsOpenProfileFromMem(ctx,buffer,input_size);
 }
 
 /* Get ICC Profile handle from file ptr */
@@ -323,7 +327,7 @@ gscms_get_profile_handle_file(const char *filename, gs_memory_t *mem)
 {
     cmsContext ctx = gs_lib_ctx_get_cms_context(mem);
 
-    return cmsOpenProfileFromFileTHR(ctx, filename, "r");
+    return cmsOpenProfileFromFile(ctx, filename, "r");
 }
 
 /* Transform an entire buffer */
@@ -337,12 +341,12 @@ gscms_transform_color_buffer(gx_device *dev, gsicc_link_t *icclink,
     cmsHTRANSFORM hTransform = (cmsHTRANSFORM)link_handle->hTransform;
     cmsUInt32Number dwInputFormat, dwOutputFormat, num_src_lcms, num_des_lcms;
     int  hasalpha, planarIN, planarOUT, numbytesIN, numbytesOUT, big_endianIN, big_endianOUT;
-    int k, needed_flags = 0;
+    int needed_flags = 0;
     unsigned char *inputpos, *outputpos;
     cmsContext ctx = gs_lib_ctx_get_cms_context(icclink->memory);
 
 #if DUMP_CMS_BUFFER
-    FILE *fid_in, *fid_out;
+    gp_file *fid_in, *fid_out;
 #endif
     /* Although little CMS does  make assumptions about data types in its
        transformations we can change it after the fact by cloning from any
@@ -445,84 +449,17 @@ gscms_transform_color_buffer(gx_device *dev, gsicc_link_t *icclink,
         }
     }
 
-    /* littleCMS knows nothing about word boundarys.  As such, we need to do
-       this row by row adjusting for our stride.  Output buffer must already
-       be allocated. ToDo:  Check issues with plane and row stride and word
-       boundry */
     inputpos = (byte *) inputbuffer;
     outputpos = (byte *) outputbuffer;
-    if(input_buff_desc->is_planar){
-        /* Determine if we can do this in one operation or if we have to break
-           it up.  Essentially if the width * height = plane_stride then yes.  If
-           we are doing some subsection of a plane then no. */
-        if (input_buff_desc->num_rows * input_buff_desc->pixels_per_row ==
-            input_buff_desc->plane_stride  &&
-            output_buff_desc->num_rows * output_buff_desc->pixels_per_row ==
-            output_buff_desc->plane_stride) {
-            /* Do entire buffer.*/
-            cmsDoTransform(ctx, hTransform, inputpos, outputpos,
-                           input_buff_desc->plane_stride);
-        } else {
-            /* We have to do this row by row, with memory transfers */
-            byte *temp_des, *temp_src;
-            int source_size = input_buff_desc->bytes_per_chan *
-                              input_buff_desc->pixels_per_row;
+    cmsDoTransformLineStride(ctx, hTransform,
+        inputpos, outputpos, input_buff_desc->pixels_per_row,
+        input_buff_desc->num_rows, input_buff_desc->row_stride,
+        output_buff_desc->row_stride, input_buff_desc->plane_stride,
+        output_buff_desc->plane_stride);
 
-            int des_size = output_buff_desc->bytes_per_chan *
-                           output_buff_desc->pixels_per_row;
-            int y, i;
-
-            temp_src = (byte*)gs_alloc_bytes(icclink->memory->non_gc_memory,
-                                              source_size * input_buff_desc->num_chan,
-                                              "gscms_transform_color_buffer");
-            if (temp_src == NULL)
-                return_error(gs_error_VMerror);
-            temp_des = (byte*) gs_alloc_bytes(icclink->memory->non_gc_memory,
-                                              des_size * output_buff_desc->num_chan,
-                                              "gscms_transform_color_buffer");
-            if (temp_des == NULL)
-                return_error(gs_error_VMerror);
-            for (y = 0; y < input_buff_desc->num_rows; y++) {
-                byte *src_cm = temp_src;
-                byte *src_buff = inputpos;
-                byte *des_cm = temp_des;
-                byte *des_buff = outputpos;
-
-                /* Put into planar temp buffer */
-                for (i = 0; i < input_buff_desc->num_chan; i ++) {
-                    memcpy(src_cm, src_buff, source_size);
-                    src_cm += source_size;
-                    src_buff += input_buff_desc->plane_stride;
-                }
-                /* Transform */
-                cmsDoTransform(ctx, hTransform, temp_src, temp_des,
-                               input_buff_desc->pixels_per_row);
-                /* Get out of temp planar buffer */
-                for (i = 0; i < output_buff_desc->num_chan; i ++) {
-                    memcpy(des_buff, des_cm, des_size);
-                    des_cm += des_size;
-                    des_buff += output_buff_desc->plane_stride;
-                }
-                inputpos += input_buff_desc->row_stride;
-                outputpos += output_buff_desc->row_stride;
-            }
-            gs_free_object(icclink->memory->non_gc_memory, temp_src,
-                           "gscms_transform_color_buffer");
-            gs_free_object(icclink->memory->non_gc_memory, temp_des,
-                           "gscms_transform_color_buffer");
-        }
-    } else {
-        /* Do row by row. */
-        for(k = 0; k < input_buff_desc->num_rows ; k++){
-            cmsDoTransform(ctx, hTransform, inputpos, outputpos,
-                           input_buff_desc->pixels_per_row);
-            inputpos += input_buff_desc->row_stride;
-            outputpos += output_buff_desc->row_stride;
-        }
-    }
 #if DUMP_CMS_BUFFER
-    fid_in = gp_fopen("CM_Input.raw","ab");
-    fid_out = gp_fopen("CM_Output.raw","ab");
+    fid_in = gp_fopen(icclink->memory,"CM_Input.raw","ab");
+    fid_out = gp_fopen(icclink->memory,"CM_Output.raw","ab");
     fwrite((unsigned char*) inputbuffer,sizeof(unsigned char),
                             input_buff_desc->row_stride,fid_in);
     fwrite((unsigned char*) outputbuffer,sizeof(unsigned char),
@@ -735,7 +672,7 @@ gscms_get_link(gcmmhprofile_t  lcms_srchandle, gcmmhprofile_t lcms_deshandle,
                                                          "gscms_transform_color_buffer");
     if (link_handle == NULL)
         return NULL;
-    link_handle->hTransform = cmsCreateTransformTHR(ctx, lcms_srchandle, src_data_type,
+    link_handle->hTransform = cmsCreateTransform(ctx, lcms_srchandle, src_data_type,
                                                     lcms_deshandle, des_data_type,
                                                     rendering_params->rendering_intent,
                                                     flag | cmm_flags);
@@ -853,7 +790,7 @@ gscms_get_link_proof_devlink(gcmmhprofile_t lcms_srchandle,
         }
 
         /* Use relative colorimetric here */
-        link_handle->hTransform = cmsCreateMultiprofileTransformTHR(ctx,
+        link_handle->hTransform = cmsCreateMultiprofileTransform(ctx,
                     hProfiles, nProfiles, src_data_type, des_data_type,
                     gsRELATIVECOLORIMETRIC, flag);
         cmsCloseProfile(ctx, src_to_proof);
@@ -912,7 +849,7 @@ gscms_get_link_proof_devlink(gcmmhprofile_t lcms_srchandle,
             || rendering_params->black_point_comp == gsBLACKPTCOMP_ON_OR) {
             flag = (flag | cmsFLAGS_BLACKPOINTCOMPENSATION);
         }
-        link_handle->hTransform =  cmsCreateMultiprofileTransformTHR(ctx,
+        link_handle->hTransform =  cmsCreateMultiprofileTransform(ctx,
                                        hProfiles, nProfiles, src_data_type,
                                        des_data_type, rendering_params->rendering_intent, flag);
     }
@@ -935,10 +872,14 @@ gscms_create(gs_memory_t *memory)
         return_error(gs_error_VMerror);
 
 #ifdef USE_LCMS2_LOCKING
-    cmsPluginTHR(ctx, (void *)&gs_cms_mutexhandler);
+    cmsPlugin(ctx, (void *)&gs_cms_mutexhandler);
 #endif
 
-    cmsSetLogErrorHandlerTHR(ctx, gscms_error);
+#ifdef WITH_CAL
+    cmsPlugin(ctx, cal_cms_extensions());
+#endif
+
+    cmsSetLogErrorHandler(ctx, gscms_error);
     gs_lib_ctx_set_cms_context(memory, ctx);
     return 0;
 }
@@ -1058,7 +999,7 @@ gscms_get_name2device_link(gsicc_link_t *icclink,
 
     /* Create the transform */
     /* ToDo:  Adjust rendering intent */
-    hTransform = cmsCreateProofingTransformTHR(ctx,
+    hTransform = cmsCreateProofingTransform(ctx,
                                             lcms_srchandle, TYPE_NAMED_COLOR_INDEX,
                                             lcms_deshandle, TYPE_CMYK_8,
                                             lcms_proofhandle,INTENT_PERCEPTUAL,
