@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -149,6 +149,9 @@ gx_device_text_begin(gx_device * dev, gs_gstate * pgs,
 static int
 gs_text_enum_init_dynamic(gs_text_enum_t *pte, gs_font *font)
 {
+    uint operation = pte->text.operation;
+    bool propagate_charpath = (operation & TEXT_DO_DRAW) != 0;
+
     pte->current_font = font;
     pte->index = 0;
     pte->xy_index = 0;
@@ -157,6 +160,24 @@ gs_text_enum_init_dynamic(gs_text_enum_t *pte, gs_font *font)
     pte->device_disabled_grid_fitting = 0;
     pte->outer_CID = GS_NO_GLYPH;
     pte->single_byte_space = false;
+    pte->cc = NULL;
+
+    /* We need to set the charpath_flag, as the PCL interpreter calls the
+     * graphics lib to do some measurement operations, which relies on the
+     * charpath_flag. See bug 700577 for more details. */
+    if (operation & TEXT_DO_ANY_CHARPATH)
+        pte->charpath_flag =
+            (operation & TEXT_DO_FALSE_CHARPATH ? cpm_false_charpath :
+             operation & TEXT_DO_TRUE_CHARPATH ? cpm_true_charpath :
+             operation & TEXT_DO_FALSE_CHARBOXPATH ? cpm_false_charboxpath :
+             operation & TEXT_DO_TRUE_CHARBOXPATH ? cpm_true_charboxpath :
+             operation & TEXT_DO_CHARWIDTH ? cpm_charwidth :
+             cpm_show /* can't happen */ );
+    else
+        pte->charpath_flag =
+        (propagate_charpath ? (pte->pgs ? pte->pgs->in_charpath : 0) : cpm_show);
+    pte->is_pure_color = pte->pgs ? gs_color_writes_pure(pte->pgs) : 0;
+
     return font->procs.init_fstack(pte, font);
 }
 int
@@ -264,6 +285,7 @@ gs_text_begin(gs_gstate * pgs, const gs_text_params_t * text,
        to compute a text width.
        Note : FontType 3 throws error in setcachedevice. */
     if (pgs->font->FontType != ft_user_defined &&
+        pgs->font->FontType != ft_PDF_user_defined &&
         pgs->font->FontType != ft_GL2_stick_user_defined &&
         pgs->font->FontType != ft_PCL_user_defined &&
         pgs->font->FontMatrix.xx == 0 && pgs->font->FontMatrix.xy == 0 &&
