@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -71,33 +71,73 @@ clist_bitmap_bytes(uint width_bits, uint height, int compression_mask,
  * necessary.  We require height >= 1, raster >= bitmap_raster(width_bits).
  */
 static int
+go_process(stream_state * st, stream_cursor_read *pr, stream_cursor_write *pw, bool end)
+{
+    int status = (*st->templat->process) (st, pr, pw, end);
+    if (status)
+        return status;
+    /* We don't attempt to handle compressors that */
+    /* require >1 input byte to make progress. */
+    if (pr->ptr != pr->limit)
+        return -1;
+    return 0;
+}
+static byte zeros[1<<align_bitmap_mod];
+static int
 cmd_compress_bitmap(stream_state * st, const byte * data, uint width_bits,
                     uint raster, uint height, stream_cursor_write * pw)
 {
     uint width_bytes = bitmap_raster(width_bits);
     int status = 0;
     stream_cursor_read r;
+    stream_cursor_read r2;
+    uint whole_bytes = width_bits>>3;
+    uint mask = (0xff00>>(width_bits & 7)) & 0xff;
+    uint padding = width_bytes - ((width_bits+7)>>3);
 
     r.ptr = data - 1;
-    if (raster == width_bytes) {
+    if (raster == whole_bytes) {
         r.limit = r.ptr + raster * height;
         status = (*st->templat->process) (st, &r, pw, true);
     } else {			/* Compress row-by-row. */
         uint y;
 
-        for (y = 1; (r.limit = r.ptr + width_bytes), y < height; ++y) {
-            status = (*st->templat->process) (st, &r, pw, false);
+        for (y = height-1; (r.limit = r.ptr + whole_bytes), y > 0; y--) {
+            status = go_process(st, &r, pw, false);
             if (status)
                 break;
-            if (r.ptr != r.limit) {	/* We don't attempt to handle compressors that */
-                /* require >1 input byte to make progress. */
-                status = -1;
-                break;
+            if (mask) {
+                byte b = r.ptr[1] & mask;
+                r2.limit = &b;
+                r2.ptr = r2.limit-1;
+
+                status = go_process(st, &r2, pw, false);
+                if (status)
+                    break;
+            }
+            if (padding) {
+                r2.ptr = zeros - 1;
+                r2.limit = r2.ptr + padding;
+                status = go_process(st, &r2, pw, false);
+                if (status)
+                    break;
+            }
+            r.ptr += (int)(raster - whole_bytes);
+        }
+        if (status == 0) {
+            status = go_process(st, &r, pw, padding == 0 && mask == 0);
+            if (status == 0 && mask) {
+                byte b = r.ptr[1] & mask;
+                r2.limit = &b;
+                r2.ptr = r2.limit-1;
+                status = go_process(st, &r2, pw, padding == 0);
+            }
+            if (status == 0 && padding) {
+                r2.ptr = zeros - 1;
+                r2.limit = r2.ptr + padding;
+                status = go_process(st, &r2, pw, true);
             }
-            r.ptr += raster - width_bytes;
         }
-        if (status == 0)
-            status = (*st->templat->process) (st, &r, pw, true);
     }
     if (st->templat->release)
         (*st->templat->release) (st);
@@ -110,6 +150,7 @@ cmd_compress_bitmap(stream_state * st, const byte * data, uint width_bits,
  * Return <0 if error, otherwise the compression method.
  * A return value of gs_error_limitcheck means that the bitmap was too big
  * to fit in the command reading buffer.
+ * This won't happen if the compression_mask has allow_large_bitmap set.
  * Note that this leaves room for the command and initial arguments,
  * but doesn't fill them in.
  */
@@ -119,15 +160,14 @@ cmd_put_bits(gx_device_clist_writer * cldev, gx_clist_state * pcls,
              int compression_mask, byte ** pdp, uint * psize)
 {
     uint short_raster, full_raster;
-    uint short_size =
-    clist_bitmap_bytes(width_bits, height,
-                       compression_mask & ~cmd_mask_compress_any,
-                       &short_raster, &full_raster);
+    uint short_size = clist_bitmap_bytes(width_bits, height,
+                          compression_mask & ~cmd_mask_compress_any,
+                          &short_raster, &full_raster);
     uint uncompressed_raster;
-    uint uncompressed_size =
-    clist_bitmap_bytes(width_bits, height, compression_mask,
+    uint uncompressed_size = clist_bitmap_bytes(width_bits, height, compression_mask,
                        &uncompressed_raster, &full_raster);
-    uint max_size = data_bits_size - op_size;
+    uint max_size = (compression_mask & allow_large_bitmap) ? 0x7fffffff :
+                        data_bits_size - op_size;
     gs_memory_t *mem = cldev->memory;
     byte *dp;
     int compress = 0;
@@ -189,7 +229,7 @@ cmd_put_bits(gx_device_clist_writer * cldev, gx_clist_state * pcls,
             w.ptr = wbase;
             w.limit = w.ptr + min(wmax, short_size >> 1);
             status = cmd_compress_bitmap((stream_state *) & sstate, data,
-                                  uncompressed_raster << 3 /*width_bits */ ,
+                                  width_bits, /* was uncompressed_raster << 3, but this overruns. */
                                          raster, height, &w);
             if (status == 0) {	/* Use compressed representation. */
                 uint wcount = w.ptr - wbase;
@@ -659,13 +699,15 @@ clist_change_tile(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                 if (tiles->num_planes != 1)
                     pdepth /= tiles->num_planes;
 
+                /* put the bits, but don't restrict to a single buffer */
                 code = cmd_put_bits(cldev, pcls, ts_bits(cldev, loc.tile),
                                     tiles->rep_width * pdepth,
                                     tiles->rep_height * tiles->num_planes,
                                     loc.tile->cb_raster, rsize,
-                                    (cldev->tile_params.size.x > tiles->rep_width ?
-                                     decompress_elsewhere | decompress_spread :
-                                     decompress_elsewhere),
+                                    allow_large_bitmap |
+                                        (cldev->tile_params.size.x > tiles->rep_width ?
+                                             decompress_elsewhere | decompress_spread :
+                                             decompress_elsewhere),
                                     &dp, &csize);
 
                 if (code < 0)
@@ -750,11 +792,13 @@ clist_change_bits(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                     pdepth /= loc.tile->num_planes;
             if (loc.tile->num_bands == CHAR_ALL_BANDS_COUNT)
                 bit_pcls = NULL;
+            /* put the bits, but don't restrict to a single buffer */
             code = cmd_put_bits(cldev, bit_pcls, ts_bits(cldev, loc.tile),
                                 loc.tile->width * pdepth,
                                 loc.tile->height * loc.tile->num_planes, loc.tile->cb_raster,
                                 rsize,
-                              decompress_elsewhere | (cldev->target->BLS_force_memory ? (1 << cmd_compress_cfe) : 0),
+                                decompress_elsewhere |
+                                    (cldev->target->BLS_force_memory ? (1 << cmd_compress_cfe) : 0),
                                 &dp, &csize);
 
             if (code < 0)
