@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -87,8 +87,7 @@ s_band_read_init(stream_state * st)
     ss->b_this.band_min = 0;
     ss->b_this.band_max = 0;
     ss->b_this.pos = 0;
-    io_procs->rewind(ss->page_bfile, false, ss->page_bfname);
-    return 0;
+    return io_procs->rewind(ss->page_bfile, false, ss->page_bfname);
 }
 
 #ifdef DEBUG
