@@ -425,21 +425,6 @@ clist_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
     return 0;
 }
 
-static void update_color_use_frac_array(uchar num_colors, const frac31 *color,
-    cmd_rects_enum_t *re)
-{
-    uchar k;
-
-    if (color == NULL)
-        return;
-
-    for (k = 0; k < num_colors; k++){
-        if (color[k] != 0) {
-            re->pcls->color_usage.or |= (1<<k);
-        }
-    }
-}
-
 static inline int
 clist_write_fill_trapezoid(gx_device * dev,
     const gs_fixed_edge *left, const gs_fixed_edge *right,
@@ -517,14 +502,11 @@ clist_write_fill_trapezoid(gx_device * dev,
         } else {
             /* Even with pdcolor NULL, we may still have colors packed in
                c0, c1, c2 or c3 */
-            update_color_use_frac_array(dev->color_info.num_components, c0, &re);
-            update_color_use_frac_array(dev->color_info.num_components, c1, &re);
-            update_color_use_frac_array(dev->color_info.num_components, c2, &re);
-            update_color_use_frac_array(dev->color_info.num_components, c3, &re);
+            re.pcls->color_usage.or |= gx_color_usage_all(cdev);
             code = 0;
         }
         if (code >= 0) {
-            /* Dont't want to shorten the trapezoid by the band boundary,
+            /* Don't want to shorten the trapezoid by the band boundary,
                keeping in mind a further optimization with writing same data to all bands. */
             code = cmd_write_trapezoid_cmd(cdev, re.pcls, cmd_opv_fill_trapezoid, left, right,
                                       ybot, ytop, options, fa, c0, c1, c2, c3);
@@ -920,6 +902,8 @@ clist_copy_planes(gx_device * dev,
     gx_bitmap_id orig_id = id;
     cmd_rects_enum_t re;
     int bpc = dev->color_info.depth / dev->color_info.num_components;
+    int pix_in_byte = bpc <= 8 ? 8/bpc : 1;
+    int byte_in_pix = bpc >= 8 ? bpc/8 : 1;
 
     if (rwidth <= 0 || rheight <= 0)
         return 0;
@@ -956,9 +940,9 @@ clist_copy_planes(gx_device * dev,
         byte *dp, *dp2;
         uint csize;
         byte op = (byte) cmd_op_copy_mono_planes;
-        int dx = data_x % (8/bpc);
+        int dx = data_x % pix_in_byte;
         int w1 = dx + rwidth;
-        const byte *row = data + (re.y - y0) * raster + (data_x / (8/bpc));
+        const byte *row = data + (re.y - y0) * raster + (data_x * byte_in_pix / pix_in_byte);
         int bytes_row = ((w1*bpc+7)/8 + 7) & ~7;
         int maxheight = data_bits_size / bytes_row / cdev->color_info.num_components;
         int plane;
@@ -1557,7 +1541,7 @@ clist_strip_copy_rop2(gx_device * dev,
                         tile_with_id.id = gs_next_ids(dev->memory, 1);
                         tiles = &tile_with_id;
                     }
-                    code = clist_change_tile(cdev, re.pcls, tiles,
+                    code = clist_change_bits(cdev, re.pcls, tiles,
                                             (tcolors != 0 ? 1 :
                                                  cdev->clist_color_info.depth));
                     if (code < 0 && !(code == gs_error_limitcheck))
