@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -59,9 +59,9 @@ static RELOC_PTRS_WITH(stream_reloc_ptrs, stream *st)
             RELOC_VAR(st->cbuf);
         reloc = cbuf_old - st->cbuf;
         /* Relocate the other buffer pointers. */
-        st->srptr -= reloc;
-        st->srlimit -= reloc;	/* same as swptr */
-        st->swlimit -= reloc;
+        st->cursor.r.ptr -= reloc;
+        st->cursor.r.limit -= reloc;	/* same as swptr */
+        st->cursor.w.limit -= reloc;
     }
     RELOC_VAR(st->strm);
     RELOC_VAR(st->prev);
@@ -159,8 +159,8 @@ s_std_init(register stream * s, byte * ptr, uint len, const stream_procs * pp,
 {
     s->templat = &s_no_template;
     s->cbuf = ptr;
-    s->srptr = s->srlimit = s->swptr = ptr - 1;
-    s->swlimit = ptr - 1 + len;
+    s->cursor.r.ptr = s->cursor.r.limit = s->cursor.w.ptr = ptr - 1;
+    s->cursor.w.limit = ptr - 1 + len;
     s->end_status = 0;
     s->foreign = 0;
     s->modes = modes;
@@ -227,14 +227,14 @@ s_std_null(stream * s)
 void
 s_std_read_reset(stream * s)
 {
-    s->srptr = s->srlimit = s->cbuf - 1;
+    s->cursor.r.ptr = s->cursor.r.limit = s->cbuf - 1;
 }
 
 /* Discard the contents of the buffer when writing. */
 void
 s_std_write_reset(stream * s)
 {
-    s->swptr = s->cbuf - 1;
+    s->cursor.w.ptr = s->cbuf - 1;
 }
 
 /* Flush data to end-of-file when reading. */
@@ -242,7 +242,7 @@ int
 s_std_read_flush(stream * s)
 {
     while (1) {
-        s->srptr = s->srlimit = s->cbuf - 1;
+        s->cursor.r.ptr = s->cursor.r.limit = s->cbuf - 1;
         if (s->end_status)
             break;
         s_process_read_buf(s);
@@ -393,7 +393,7 @@ stell(stream * s)
      * The stream might have been closed, but the position
      * is still meaningful in this case.
      */
-    const byte *ptr = (s_is_writing(s) ? s->swptr : s->srptr);
+    const byte *ptr = (s_is_writing(s) ? s->cursor.w.ptr : s->cursor.r.ptr);
 
     return (ptr == 0 ? 0 : ptr + 1 - s->cbuf) + s->position;
 }
@@ -456,7 +456,7 @@ spgetcc(register stream * s, bool close_at_eod)
     int min_left = sbuf_min_left(s);
 
     while (status = s->end_status,
-           left = s->srlimit - s->srptr,
+           left = s->cursor.r.limit - s->cursor.r.ptr,
            left <= min_left && status >= 0
         )
         s_process_read_buf(s);
@@ -474,7 +474,7 @@ spgetcc(register stream * s, bool close_at_eod)
         }
         return status;
     }
-    return *++(s->srptr);
+    return *++(s->cursor.r.ptr);
 }
 
 /* Implementing sputc when the buffer is full, */
@@ -486,7 +486,7 @@ spputc(register stream * s, byte b)
         if (s->end_status)
             return s->end_status;
         if (!sendwp(s)) {
-            *++(s->swptr) = b;
+            *++(s->cursor.w.ptr) = b;
             return b;
         }
         s_process_write_buf(s, false);
@@ -503,9 +503,9 @@ sungetc(register stream * s, byte c)
        unread from stdin, ever.
      */
     if (s->cbuf == NULL || !s_is_reading(s) ||
-        s->srptr < s->cbuf || *(s->srptr) != c)
+        s->cursor.r.ptr < s->cbuf || *(s->cursor.r.ptr) != c)
         return ERRC;
-    s->srptr--;
+    s->cursor.r.ptr--;
     return 0;
 }
 
@@ -523,10 +523,10 @@ sgets(stream * s, byte * buf, uint nmax, uint * pn)
     while (cw.ptr < cw.limit) {
         int left;
 
-        if ((left = s->srlimit - s->srptr) > min_left) {
-            s->srlimit -= min_left;
+        if ((left = s->cursor.r.limit - s->cursor.r.ptr) > min_left) {
+            s->cursor.r.limit -= min_left;
             stream_move(&s->cursor.r, &cw);
-            s->srlimit += min_left;
+            s->cursor.r.limit += min_left;
         } else {
             uint wanted = cw.limit - cw.ptr;
             int c;
@@ -550,7 +550,7 @@ sgets(stream * s, byte * buf, uint nmax, uint * pn)
                  * update position.  However, we need to reset the read
                  * cursor to indicate that there is no data in the buffer.
                  */
-                s->srptr = s->srlimit = s->cbuf - 1;
+                s->cursor.r.ptr = s->cursor.r.limit = s->cbuf - 1;
                 s->position += cw.ptr - wptr;
                 if (status <= 0 || cw.ptr == cw.limit)
                     break;
@@ -577,13 +577,13 @@ sputs(register stream * s, const byte * str, uint wlen, uint * pn)
 
     if (status >= 0)
         while (len > 0) {
-            uint count = s->swlimit - s->swptr;
+            uint count = s->cursor.w.limit - s->cursor.w.ptr;
 
             if (count > 0) {
                 if (count > len)
                     count = len;
-                memcpy(s->swptr + 1, str, count);
-                s->swptr += count;
+                memcpy(s->cursor.w.ptr + 1, str, count);
+                s->cursor.w.ptr += count;
                 str += count;
                 len -= count;
             } else {
@@ -624,7 +624,7 @@ spskip(register stream * s, gs_offset_t nskip, gs_offset_t *pskipped)
         int status;
 
         n -= sbufavailable(s);
-        s->srptr = s->srlimit;
+        s->cursor.r.ptr = s->cursor.r.limit;
         if (s->end_status) {
             *pskipped = nskip - n;
             return s->end_status;
@@ -637,7 +637,7 @@ spskip(register stream * s, gs_offset_t nskip, gs_offset_t *pskipped)
         --n;
     }
     /* Note that if min_left > 0, n < 0 is possible; this is harmless. */
-    s->srptr += n;
+    s->cursor.r.ptr += n;
     *pskipped = nskip;
     return 0;
 }
@@ -1023,19 +1023,19 @@ sread_string(register stream *s, const byte *ptr, uint len)
     s->cbuf_string.data = (byte *)ptr;
     s->cbuf_string.size = len;
     s->end_status = EOFC;
-    s->srlimit = s->swlimit;
+    s->cursor.r.limit = s->cursor.w.limit;
 }
 /* Initialize a reusable stream for reading a string. */
 static void
 s_string_reusable_reset(stream *s)
 {
-    s->srptr = s->cbuf - 1;	/* just reset to the beginning */
-    s->srlimit = s->srptr + s->bsize;  /* might have gotten reset */
+    s->cursor.r.ptr = s->cbuf - 1;	/* just reset to the beginning */
+    s->cursor.r.limit = s->cursor.r.ptr + s->bsize;  /* might have gotten reset */
 }
 static int
 s_string_reusable_flush(stream *s)
 {
-    s->srptr = s->srlimit = s->cbuf + s->bsize - 1;  /* just set to the end */
+    s->cursor.r.ptr = s->cursor.r.limit = s->cbuf + s->bsize - 1;  /* just set to the end */
     return 0;
 }
 void
@@ -1071,9 +1071,9 @@ s_string_read_seek(register stream * s, gs_offset_t pos)
 {
     if (pos < 0 || pos > s->bsize)
         return ERRC;
-    s->srptr = s->cbuf + pos - 1;
+    s->cursor.r.ptr = s->cbuf + pos - 1;
     /* We might be seeking after a reusable string reached EOF. */
-    s->srlimit = s->cbuf + s->bsize - 1;
+    s->cursor.r.limit = s->cbuf + s->bsize - 1;
     /*
      * When the file reaches EOF,
      * stream_compact sets s->position to its end.
@@ -1106,7 +1106,7 @@ s_string_write_seek(register stream * s, gs_offset_t pos)
 {
     if (pos < 0 || pos > s->bsize)
         return ERRC;
-    s->swptr = s->cbuf + pos - 1;
+    s->cursor.w.ptr = s->cbuf + pos - 1;
     return 0;
 }
 
