@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -170,6 +170,18 @@ cmykog_open(gx_device * pdev)
   dev->color_info.separable_and_linear = GX_CINFO_SEP_LIN;
   dev->icc_struct->supports_devn = true;
 
+  /* This device is a DeviceN color based device but unlike psdcmyk and
+     tiffsep it can't change its color characteristics.  It remains as 
+     a 6 color cmykog device.  Postsript files can change the maxseparations 
+     (example Bug 696600 29-07E.PS)  The gdevdevn code will, if it 
+     detects a change occured, close the device and reopen (see
+     devn_printer_put_params) possibly resetting the color info.  tiffsep
+     and psdcmyk, when reopened, will  reset their color characteristics based upon
+     the content of the page, avoiding any issues upon the reopen.  Here we
+     will need to make sure to set the color information to avoid problems. */
+  pdev->color_info.max_components = 6;
+  pdev->color_info.depth = pdev->color_info.num_components * 8;
+
   /* For the planar device we need to set up the bit depth of each plane.
    * For other devices this is handled in check_device_separable where
    * we compute the bit shift for the components etc. */
@@ -216,7 +228,7 @@ cmykog_dev_spec_op(gx_device *dev_, int op, void *data, int datasize)
   if (op == gxdso_supports_devn) {
     return true;
   }
-  return gx_default_dev_spec_op(dev_, op, data, datasize);
+  return gdev_prn_dev_spec_op(dev_, op, data, datasize);
 }
 
 /*
@@ -282,7 +294,7 @@ cmykog_put_params(gx_device * pdev, gs_param_list * plist)
  * passed to every process_page function:
  */
 typedef struct cmykog_process_arg_s {
-  FILE *spot_file[GX_DEVICE_COLOR_MAX_COMPONENTS];
+  gp_file *spot_file[GX_DEVICE_COLOR_MAX_COMPONENTS];
   char spot_name[GX_DEVICE_COLOR_MAX_COMPONENTS][gp_file_name_sizeof];
   int dev_raster;
 } cmykog_process_arg_t;
@@ -539,11 +551,11 @@ cmykog_process(void *arg_, gx_device *dev_, gx_device *bdev, const gs_int_rect *
 }
 
 static void
-write_plane(FILE *file, byte *data, int w, int h, int raster)
+write_plane(gp_file *file, byte *data, int w, int h, int raster)
 {
 #ifndef NO_OUTPUT
   for (; h > 0; h--) {
-      fwrite(data, 1, w, file);
+      gp_fwrite(data, 1, w, file);
       data += raster;
   }
 #endif
@@ -561,7 +573,7 @@ static const char empty[64] = {
 };
 
 static void
-write_empty_plane(FILE *file, int w, int h)
+write_empty_plane(gp_file *file, int w, int h)
 {
 #ifndef NO_OUTPUT
   w *= h;
@@ -569,7 +581,7 @@ write_empty_plane(FILE *file, int w, int h)
     h = w;
     if (h > 64)
         h = 64;
-    fwrite(empty, 1, h, file);
+    gp_fwrite(empty, 1, h, file);
     w -= h;
   }
 #endif
@@ -604,7 +616,7 @@ cmykog_output(void *arg_, gx_device *dev_, void *buffer_)
 /* This, finally is the main entry point that triggers printing for the
  * device. */
 static int
-cmykog_print_page(gx_device_printer * pdev, FILE * prn_stream)
+cmykog_print_page(gx_device_printer * pdev, gp_file * prn_stream)
 {
   gx_device_cmykog * pdevn = (gx_device_cmykog *) pdev;
   int ncomp = pdevn->color_info.num_components;
@@ -674,10 +686,10 @@ cmykog_print_page(gx_device_printer * pdev, FILE * prn_stream)
     for (i = 1; i < ncomp; i++) {
       char tmp[4096];
       int n;
-      fseek(arg->spot_file[i], 0, SEEK_SET);
-      while (!feof(arg->spot_file[i])) {
-        n = fread(tmp, 1, 4096, arg->spot_file[i]);
-        fwrite(tmp, 1, n, prn_stream);
+      gp_fseek(arg->spot_file[i], 0, SEEK_SET);
+      while (!gp_feof(arg->spot_file[i])) {
+        n = gp_fread(tmp, 1, 4096, arg->spot_file[i]);
+        gp_fwrite(tmp, 1, n, prn_stream);
       }
     }
     /* If Ghostscript knows that all 6 planes aren't used, it may
@@ -693,7 +705,7 @@ prn_done:
     /* Close the temporary files. */
     for(i = 1; i < ncomp; i++) {
       if (arg->spot_file[i] != NULL)
-        fclose(arg->spot_file[i]);
+        gp_fclose(arg->spot_file[i]);
       if(arg->spot_name[i][0])
         unlink(arg->spot_name[i]);
     }
