@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -19,7 +19,10 @@
 #ifndef gdevtfax_INCLUDED
 #  define gdevtfax_INCLUDED
 
-int gdev_fax_print_page_stripped(gx_device_printer *pdev, FILE *prn_stream,
+#include "gxclist.h"
+#include "scfx.h"
+
+int gdev_fax_print_page_stripped(gx_device_printer *pdev, gp_file *prn_stream,
                                  stream_CFE_state *ss, long rows_per_strip);
 
 #endif /* gdevtfax_INCLUDED */
