@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -1006,11 +1006,26 @@ ex:
     *x1 = x;
 }
 
+static int
+cmpbits(const byte *base, const byte *base2, int w)
+{
+    int code;
+
+    code = memcmp(base, base2, w>>3);
+    if (code)
+        return code;
+    base += w>>3;
+    base2 += w>>3;
+    w &= 7;
+    if (w == 0)
+        return 0;
+    return ((*base ^ *base2) & (0xff00>>w));
+}
+
 static void
 compute_subimage(int width, int height, int raster, byte *base,
                  int x0, int y0, long MaxClipPathSize, int *x1, int *y1)
 {
-    int bytes = (width + 7) / 8;
     /* Returns a semiopen range : [x0:x1)*[y0:y1). */
     if (x0 != 0) {
         long count;
@@ -1038,7 +1053,7 @@ compute_subimage(int width, int height, int raster, byte *base,
             count1 -= count;
             yy = y + 1;
             for (; yy < height; yy++)
-                if (memcmp(base + raster * y, base + raster * yy, bytes))
+                if (cmpbits(base + raster * y, base + raster * yy, width))
                     break;
             y = yy;
 
@@ -1097,14 +1112,14 @@ static int
 mask_to_clip(gx_device_pdf *pdev, int width, int height,
              int raster, byte *base, int x0, int y0, int x1, int y1)
 {
-    int y, yy, code = 0, bytes = (width + 7) / 8;
+    int y, yy, code = 0;
     bool has_segments = false;
 
     for (y = y0; y < y1 && code >= 0;) {
         yy = y + 1;
         if (x0 == 0) {
         for (; yy < y1; yy++)
-            if (memcmp(base + raster * y, base + raster * yy, bytes))
+            if (cmpbits(base + raster * y, base + raster * yy, width))
                 break;
         }
         code = image_line_to_clip(pdev, base + raster * y, x0, x1, y, yy, has_segments);
