@@ -337,7 +337,7 @@ typedef struct pdf_page_s {
  */
 typedef struct pdf_temp_file_s {
     char file_name[gp_file_name_sizeof];
-    FILE *file;
+    gp_file *file;
     stream *strm;
     byte *strm_buf;
     stream *save_strm;                /* save pdev->strm while writing here */
@@ -411,7 +411,7 @@ typedef struct share_hint_stream_s {
 } shared_hint_stream_t;
 
 typedef struct pdf_linearisation_s {
-    FILE *sfile;
+    gp_file *sfile;
     pdf_temp_file_t Lin_File;
     char HintBuffer[256];
     unsigned char HintBits;
@@ -786,6 +786,19 @@ struct gx_device_pdf_s {
     int PatternDepth;
     gs_matrix AccumulatedPatternMatrix;
 
+    /* Normally the PDF itnerpreter doesn't call execform to pass a From XObject to
+     * pdfwrite, but it *does* create forms (and increment FormDepth) for transparent
+     * Groups, which are handled as Form XObjects. Because of the way that Pattterns work,
+     * the first pattern after a Form uses the Form co-ordinate system, but if the Pattern
+     * itself uses Patterns, then the nested Pattern needs to use the co-ordinate system
+     * of the parent Pattern. Unless, of course, we have another Form!
+     * So essentially we need to know the Pattern depth, since the last form was executed.
+     * If its 0, then we want to apply the current CTM, if its more than that then we want
+     * to undo the pdfwrite scaling (see gdevpdfv.c, pdf_store_pattern1_params() at about line
+     * 239.
+     */
+    int PatternsSinceForm;
+
     /* Accessories */
     cos_dict_t *substream_Resources;     /* Substream resources */
     gs_color_space_index pcm_color_info_index; /* Index of the ProcessColorModel space. */
@@ -983,6 +996,10 @@ void pdf_reset_text(gx_device_pdf *pdev);
 
 /* ------ Document ------ */
 
+/* Utility functions to write args into the file as comments */
+int pdfwrite_fwrite_args_comment(gx_device_pdf *pdev, gp_file *f);
+int pdfwrite_write_args_comment(gx_device_pdf *pdev, stream *s);
+
 /* Write a DSC compliant header to the file */
 int ps2write_dsc_header(gx_device_pdf * pdev, int pages);
 
@@ -1141,8 +1158,8 @@ int pdf_free_resource_objects(gx_device_pdf *pdev, pdf_resource_type_t rtype);
 int pdf_store_page_resources(gx_device_pdf *pdev, pdf_page_t *page, bool clear_usage);
 
 /* Copy data from a temporary file to a stream. */
-int pdf_copy_data(stream *s, FILE *file, gs_offset_t count, stream_arcfour_state *ss);
-int pdf_copy_data_safe(stream *s, FILE *file, gs_offset_t position, long count);
+int pdf_copy_data(stream *s, gp_file *file, gs_offset_t count, stream_arcfour_state *ss);
+int pdf_copy_data_safe(stream *s, gp_file *file, gs_offset_t position, long count);
 
 /* Add the encryption filter. */
 int pdf_begin_encrypt(gx_device_pdf * pdev, stream **s, gs_id object_id);
