@@ -714,6 +714,7 @@ pclxl_write_image_data_RLE(gx_device_pclxl * xdev, const byte * base,
     uint num_bytes = ROUND_UP(width_bytes, 4) * height;
     bool compress = num_bytes >= 8;
     int i;
+    int code;
 
     /* cannot handle data_bit not multiple of 8, but we don't invoke this routine that way */
     int offset = data_bit >> 3;
@@ -752,19 +753,20 @@ pclxl_write_image_data_RLE(gx_device_pclxl * xdev, const byte * base,
             r.ptr = data + i * raster - 1;
             r.limit = r.ptr + width_bytes;
             if ((*s_RLE_template.process)
-                ((stream_state *) & rlstate, &r, &w, true) != 0 ||
+                ((stream_state *) & rlstate, &r, &w, false) != 0 ||
                 r.ptr != r.limit)
                 goto ncfree;
             r.ptr = (const byte *)"\000\000\000\000\000";
             r.limit = r.ptr + (-(int)width_bytes & 3);
             if ((*s_RLE_template.process)
-                ((stream_state *) & rlstate, &r, &w, true) != 0 ||
+                ((stream_state *) & rlstate, &r, &w, false) != 0 ||
                 r.ptr != r.limit)
                 goto ncfree;
         }
         r.ptr = r.limit;
-        if ((*s_RLE_template.process)
-            ((stream_state *) & rlstate, &r, &w, true) != 0)
+        code = (*s_RLE_template.process)
+            ((stream_state *) & rlstate, &r, &w, true);
+        if (code != EOFC && code != 0)
             goto ncfree;
         {
             uint count = w.ptr + 1 - buf;
@@ -1339,11 +1341,18 @@ pclxl_setdash(gx_device_vector * vdev, const float *pattern, uint count,
         return_error(gs_error_limitcheck);
     else {
         uint i;
-
+        uint pattern_length = 0;
         /*
          * Astoundingly, PCL XL doesn't allow real numbers here.
          * Do the best we can.
          */
+
+        /* check if the resulting total pattern length will be 0 */
+        for (i = 0; i < count; ++i)
+            pattern_length += (uint) (pattern[i]);
+        if (pattern_length == 0)
+            return_error(gs_error_rangecheck);
+        
         spputc(s, pxt_uint16_array);
         px_put_ub(s, (byte) count);
         for (i = 0; i < count; ++i)
@@ -1657,7 +1666,7 @@ pclxl_output_page(gx_device * dev, int num_copies, int flush)
     spputc(s, pxtEndPage);
     sflush(s);
     pclxl_page_init(xdev);
-    if (ferror(xdev->file))
+    if (gp_ferror(xdev->file))
         return_error(gs_error_ioerror);
     if ((code = gx_finish_output_page(dev, num_copies, flush)) < 0)
         return code;
@@ -1678,12 +1687,12 @@ static int
 pclxl_close_device(gx_device * dev)
 {
     gx_device_pclxl *const xdev = (gx_device_pclxl *) dev;
-    FILE *file = xdev->file;
+    gp_file *file = xdev->file;
 
     if (xdev->strm != NULL)
         sflush(xdev->strm);
     if (xdev->in_page)
-        fputc(pxtEndPage, file);
+        gp_fputc(pxtEndPage, file);
     px_write_file_trailer(file);
     return gdev_vector_close_file((gx_device_vector *) dev);
 }
@@ -1971,9 +1980,10 @@ pclxl_strip_copy_rop(gx_device * dev, const byte * sdata, int sourcex,
                      int x, int y, int width, int height,
                      int phase_x, int phase_y, gs_logical_operation_t lop)
 {
+    lop = lop_sanitize(lop);
     /* Improvements possible here using PXL ROP3
        for some combinations of args; use gx_default for now */
-    if (!rop3_uses_D(gs_transparent_rop(lop)))  /* gx_default() cannot cope with D ops */
+    if (!rop3_uses_D(lop))  /* gx_default() cannot cope with D ops */
         return gx_default_strip_copy_rop(dev, sdata, sourcex,
                                          sraster, id,
                                          scolors,
