@@ -27,6 +27,7 @@
 #include "pjparse.h"
 #include "plfont.h"
 #include "plver.h"              /* PJL_VOLUME_0 PJL_VOLUME_1 */
+#include "plmain.h"
 #include "ctype_.h"             /* for toupper() */
 #include <stdlib.h>             /* for atoi() */
 
@@ -82,7 +83,7 @@ typedef struct pjl_parser_state_s
     int line_size;
     int bytes_to_write;         /* data processing by fsdownload */
     int bytes_to_read;          /* data processed by fsupload */
-    FILE *fp;                   /* fsdownload or fsupload file */
+    gp_file *fp;                   /* fsdownload or fsupload file */
     int pos;                    /* current position in line */
     pjl_envir_var_t *defaults;  /* the default environment (i.e. set default) */
     pjl_envir_var_t *envir;     /* the pjl environment */
@@ -193,7 +194,9 @@ typedef enum
     NAME,                       /* used for pathnames */
     SIZE,                       /* size of data */
     VOLUME,                     /* volume indicator for filesystem initialization */
-    DISKLOCK
+    DISKLOCK,
+    GSSET,
+    GSSETSTRING
 } pjl_token_type_t;
 
 /* lookup table to map strings to pjl tokens */
@@ -230,6 +233,8 @@ static const pjl_lookup_table_t pjl_table[] = {
     {"COUNT", COUNT},
     {"OFFSET", OFFSET},
     {"DISKLOCK", DISKLOCK},
+    {"GSSET", GSSET},
+    {"GSSETSTRING", GSSETSTRING},
     {"", (pjl_token_type_t) 0 /* don't care */ }
 };
 
@@ -439,31 +444,35 @@ pjl_check_font_path(char *path_list, gs_memory_t * mem)
 {
     /* lookup a font path and check if any files (presumably fonts are
        present) */
-    char tmp_path[PJL_PATH_NAME_LENGTH + 1];
-    char *tmp_pathp = tmp_path, *tplast = NULL;
+    char path[PJL_PATH_NAME_LENGTH + 1];
+    char *pathp = path, *tplast = NULL;
     const char pattern[] = "*";
-    char tmp_path_and_pattern[PJL_PATH_NAME_LENGTH + 1 + 1];    /* pattern + null */
+    char path_and_pattern[PJL_PATH_NAME_LENGTH + 1 + 1];    /* pattern + null */
     char *dirname;
     char fontfilename[MAXPATHLEN + 1];
 
-    /* make a tmp copy of the colon delimited path */
-    strcpy(tmp_path, path_list);
+    /* Ignore error if the path list is too long, a single path may be
+       okay. */
+    gs_strlcpy(path, path_list, sizeof(path));
     /* for each path search for fonts.  If we find them return we only
        check if the directory resource has files without checking if
        the files are indeed fonts. */
-    while ((dirname = gs_strtok(tmp_pathp, ";", &tplast)) != NULL) {
+    while ((dirname = gs_strtok(pathp, ";", &tplast)) != NULL) {
         file_enum *fe;
 
-        strcpy(tmp_path_and_pattern, dirname);
-        strcat(tmp_path_and_pattern, pattern);
+        if (gs_strlcpy(path_and_pattern, dirname, sizeof(path_and_pattern)) >= sizeof(path_and_pattern))
+            continue;
 
-        fe = gs_enumerate_files_init(tmp_path_and_pattern,
-                                     strlen(tmp_path_and_pattern), mem);
+        if (gs_strlcat(path_and_pattern, pattern, sizeof(path_and_pattern)) >= sizeof(path_and_pattern))
+            continue;
+
+        fe = gs_enumerate_files_init(mem, path_and_pattern,
+                                     strlen(path_and_pattern));
         if (fe == NULL
             ||
-            (gs_enumerate_files_next(fe, fontfilename, PJL_PATH_NAME_LENGTH))
+            (gs_enumerate_files_next(mem, fe, fontfilename, PJL_PATH_NAME_LENGTH))
             == -1) {
-            tmp_pathp = NULL;
+            pathp = NULL;
         } else {
             /* wind through the rest of the files.  This should close
                things up as well.  All we need to do is clean up but
@@ -471,16 +480,21 @@ pjl_check_font_path(char *path_list, gs_memory_t * mem)
                directory */
             while (1) {
                 int fstatus =
-                    (int)gs_enumerate_files_next(fe, fontfilename,
+                    (int)gs_enumerate_files_next(mem, fe, fontfilename,
                                                  PJL_PATH_NAME_LENGTH);
                 /* we don't care if the file does not fit (return +1) */
                 if (fstatus == -1)
                     break;
             }
-            /* NB fix me - replace : separated path with real path.
-               We should do this elsewhere */
-            strcpy(path_list, dirname);
-            return path_list;
+            /* replace : separated path with real path.  The path list
+               is of equal or greater length than dirname, so it can't
+               fail, but we check anyway. */
+            if (strlen(path_list) < strlen(dirname))
+                return NULL;
+            else {
+                strcpy(path_list, dirname);
+                return path_list;
+            }
         }
     }
     return NULL;
@@ -590,21 +604,21 @@ pjl_verify_file_operation(pjl_parser_state_t * pst, char *fname)
 static void
 pjl_warn_exists(const gs_memory_t * mem, char *fname)
 {
-    FILE *fpdownload;
+    gp_file *fpdownload;
 
     /* issue a warning if the file exists */
-    if ((fpdownload = gp_fopen(fname, gp_fmode_rb)) != NULL) {
-        fclose(fpdownload);
+    if ((fpdownload = gp_fopen(mem, fname, gp_fmode_rb)) != NULL) {
+        gp_fclose(fpdownload);
         dmprintf1(mem, "warning file exists overwriting %s\n", fname);
     }
 }
 
 /* open a file for writing or appending */
-static FILE *
+static gp_file *
 pjl_setup_file_for_writing(pjl_parser_state_t * pst, char *pathname, int size,
                            bool append)
 {
-    FILE *fp;
+    gp_file *fp;
     char fname[MAXPATHLEN];
 
     pjl_parsed_filename_to_string(fname, pathname);
@@ -617,7 +631,7 @@ pjl_setup_file_for_writing(pjl_parser_state_t * pst, char *pathname, int size,
         strcpy(fmode, gp_fmode_wb);
         if (append)
             strcat(fmode, "+");
-        if ((fp = gp_fopen(fname, gp_fmode_wb)) == NULL) {
+        if ((fp = gp_fopen(pst->mem, fname, gp_fmode_wb)) == NULL) {
             dmprintf(pst->mem, "warning file open for writing failed\n");
             return NULL;
         }
@@ -630,7 +644,7 @@ static int
 pjl_set_fs_download_state(pjl_parser_state_t * pst, char *pathname, int size)
 {
     /* somethink is wrong if the state indicates we are already writing to a file */
-    FILE *fp =
+    gp_file *fp =
         pjl_setup_file_for_writing(pst, pathname, size, false /* append */ );
     if (fp == NULL)
         return -1;
@@ -644,7 +658,7 @@ pjl_set_fs_download_state(pjl_parser_state_t * pst, char *pathname, int size)
 static int
 pjl_set_append_state(pjl_parser_state_t * pst, char *pathname, int size)
 {
-    FILE *fp =
+    gp_file *fp =
         pjl_setup_file_for_writing(pst, pathname, size, true /* append */ );
     if (fp == NULL)
         return -1;
@@ -718,12 +732,11 @@ pjl_search_for_file(pjl_parser_state_t * pst, char *pathname, char *filename,
     /* should check length */
     strcpy(fontfilename, pathname);
     strcat(fontfilename, "/*");
-    fe = gs_enumerate_files_init(fontfilename, strlen(fontfilename),
-                                 pst->mem);
+    fe = gs_enumerate_files_init(pst->mem, fontfilename, strlen(fontfilename));
     if (fe) {
         do {
             uint fstatus =
-                gs_enumerate_files_next(fe, fontfilename,
+                gs_enumerate_files_next(pst->mem, fe, fontfilename,
                                         PJL_PATH_NAME_LENGTH);
             /* done */
             if (fstatus == ~(uint) 0)
@@ -753,12 +766,11 @@ pjl_fsdirlist(pjl_parser_state_t * pst, char *pathname, int entry, int count)
     pjl_parsed_filename_to_string(fontfilename, pathname);
     /* if this is a directory add * for the directory listing NB fix */
     strcat(fontfilename, "/*");
-    fe = gs_enumerate_files_init(fontfilename, strlen(fontfilename),
-                                 pst->mem);
+    fe = gs_enumerate_files_init(pst->mem, fontfilename, strlen(fontfilename));
     if (fe) {
         do {
             uint fstatus =
-                gs_enumerate_files_next(fe, fontfilename,
+                gs_enumerate_files_next(pst->mem, fe, fontfilename,
                                         PJL_PATH_NAME_LENGTH);
             /* done */
             if (fstatus == ~(uint) 0)
@@ -781,15 +793,15 @@ pjl_write_remaining_data(pjl_parser_state_t * pst, const byte ** pptr,
     uint avail = plimit - ptr;
     uint bytes_written = min(avail, pst->bytes_to_write);
 
-    if (fwrite(ptr, 1, bytes_written, pst->fp) != bytes_written) {
+    if (gp_fwrite(ptr, 1, bytes_written, pst->fp) != bytes_written) {
         /* try to close the file before failing */
-        fclose(pst->fp);
+        gp_fclose(pst->fp);
         pst->fp = NULL;
         return -1;
     }
     pst->bytes_to_write -= bytes_written;
     if (pst->bytes_to_write == 0) {     /* done */
-        fclose(pst->fp);
+        gp_fclose(pst->fp);
         pst->fp = NULL;
     }
     /* update stream pointer */
@@ -825,6 +837,33 @@ pjl_get_setting(pjl_parser_state_t * pst, pjl_token_type_t tok, char *token)
     return 0;
 }
 
+static int
+gs_set(pjl_parser_state_t *pst, const char *variable, const char *token, int string)
+{
+    int code;
+    int len1 = strlen(variable)+1;
+    int len2 = NULL ? 0 : strlen(token)+1;
+    char *buffer = (char *)gs_alloc_bytes(pst->mem, len1+len2, "gs_set buffer");
+
+    if (buffer == NULL)
+        return -1;
+
+    strcpy(buffer, variable);
+    if (len2) {
+        strcat(buffer, "=");
+        strcat(buffer, token);
+    }
+
+    if (string)
+        code = pl_main_set_string_param(pl_main_get_instance(pst->mem), buffer);
+    else
+        code = pl_main_set_param(pl_main_get_instance(pst->mem), buffer);
+    gs_free_object(pst->mem, buffer, "gs_set buffer");
+
+    return code;
+}
+
+
 /* parse and set up state for one line of pjl commands */
 static int
 pjl_parse_and_process_line(pjl_parser_state_t * pst)
@@ -837,10 +876,10 @@ pjl_parse_and_process_line(pjl_parser_state_t * pst)
     token = (char *)gs_alloc_bytes(pst->mem, bufsize, "working buffer for PJL parsing");
     if (token == 0)
         return -1;
-    
+
     memset(token, 0x00, bufsize);
     memset(pathname, 0x00, sizeof(pathname));
-           
+
     /* reset the line position to the beginning of the line */
     pst->pos = 0;
     /* all pjl commands start with the pjl prefix @PJL */
@@ -872,8 +911,7 @@ pjl_parse_and_process_line(pjl_parser_state_t * pst)
 
                             strcpy(variable, token);
                             if (((tok = pjl_get_token(pst, token)) == EQUAL)
-                                && (tok =
-                                    pjl_get_token(pst, token)) == SETTING) {
+                                && (tok = pjl_get_token(pst, token)) == SETTING) {
                                 /* an unfortunate side effect - we have to
                                    identify FORMLINES being set here, for
                                    the benefit of PCL, see the
@@ -885,20 +923,46 @@ pjl_parse_and_process_line(pjl_parser_state_t * pst)
                                             (char *)"on", defaults);
                                 code = pjl_set(pst, variable, token,
                                                defaults);
-                                gs_free_object(pst->mem, variable, "2nd working buffer for PJL parsing");
-                                gs_free_object(pst->mem, token, "working buffer for PJL parsing");
-                                return code;
-                            } else {
-                                gs_free_object(pst->mem, variable, "2nd working buffer for PJL parsing");
-                                gs_free_object(pst->mem, token, "working buffer for PJL parsing");
-                                return -1;      /* syntax error */
-                            }
+                            } else
+                                code = -1; /* syntax error */
+                            gs_free_object(pst->mem, variable, "2nd working buffer for PJL parsing");
+                            gs_free_object(pst->mem, token, "working buffer for PJL parsing");
+                            return code;
                         } else
                             continue;
 
                     gs_free_object(pst->mem, token, "working buffer for PJL parsing");
                     return 0;
                 }
+            case GSSET:
+            case GSSETSTRING:
+                {
+                    bool string;
+
+                    string = (tok == GSSETSTRING);
+                    code = 0;
+                    while ((tok = pjl_get_token(pst, token)) != DONE)
+                        if (tok == VARIABLE || tok == SETTING) {
+                            int len1 = strlen(token)+1;
+                            char *variable = (char *)gs_alloc_bytes(pst->mem, len1,
+                                                                    "2nd working buffer for PJL parsing");
+                            if (variable == NULL)
+                                code = -1;
+                            else {
+                                strcpy(variable, token);
+                                if (((tok = pjl_get_token(pst, token)) == EQUAL) &&
+                                    ((tok = pjl_get_token(pst, token)) == SETTING))
+                                    code = gs_set(pst, variable, token, string);
+                                else
+                                    code = gs_set(pst, variable, NULL, string);
+                                gs_free_object(pst->mem, variable, "2nd working buffer for PJL parsing");
+                            }
+                            break;
+                        }
+
+                    gs_free_object(pst->mem, token, "working buffer for PJL parsing");
+                    return code;
+                }
             case INITIALIZE:
                 /* set the user default environment to the factory default environment */
                 free_pjl_defaults(pst->mem, &pst->defaults);
@@ -1050,7 +1114,7 @@ pjl_parse_and_process_line(pjl_parser_state_t * pst)
 }
 
 /* get a file from 0: or 1: volume */
-static FILE *
+static gp_file *
 get_fp(pjl_parser_state_t * pst, char *name)
 {
     char result[MAXPATHLEN];
@@ -1064,7 +1128,7 @@ get_fp(pjl_parser_state_t * pst, char *name)
         if (result[0] == '\0')
             return 0;
     }
-    return gp_fopen(result, gp_fmode_rb);
+    return gp_fopen(pst->mem, result, gp_fmode_rb);
 }
 
 /* scan for a named resoource in the pcl sandbox 0: or 1: and return
@@ -1074,13 +1138,16 @@ long int
 pjl_get_named_resource_size(pjl_parser_state_t * pst, char *name)
 {
     long int size;
-    FILE *fp = get_fp(pst, name);
+    gp_file *fp = get_fp(pst, name);
 
     if (fp == NULL)
         return 0;
-    fseek(fp, 0L, SEEK_END);
-    size = ftell(fp);
-    fclose(fp);
+    size = gp_fseek(fp, 0L, SEEK_END);
+    if (size >= 0)
+        size = gp_ftell(fp);
+    else
+        size = 0;
+    gp_fclose(fp);
     return size;
 }
 
@@ -1091,17 +1158,18 @@ pjl_get_named_resource(pjl_parser_state * pst, char *name, byte * data)
 {
     long int size;
     int code = 0;
-    FILE *fp = get_fp(pst, name);
+    gp_file *fp = get_fp(pst, name);
 
     if (fp == NULL)
         return 0;
-    fseek(fp, 0L, SEEK_END);
-    size = ftell(fp);
-    rewind(fp);
-    if (size < 0 || (size != fread(data, 1, size, fp))) {
+    size = gp_fseek(fp, 0L, SEEK_END);
+    if (size >= 0)
+        size = gp_ftell(fp);
+    gp_rewind(fp);
+    if (size < 0 || (size != gp_fread(data, 1, size, fp))) {
         code = -1;
     }
-    fclose(fp);
+    gp_fclose(fp);
     return code;
 }
 
@@ -1723,12 +1791,24 @@ pjl_process_init(gs_memory_t * mem)
         pathlen = 0;
         if ((code = gp_getenv("PCLFONTSOURCE", (char *)0, &pathlen)) < 0) {
             char *path =
-                (char *)gs_alloc_bytes(mem, pathlen, "pjl_font_path");
+                (char *)gs_alloc_bytes(mem, pathlen + 1, "pjl_font_path");
             /* if the allocation fails we use the pjl fontsource */
             if (path == NULL)
                 pjlstate->environment_font_path = NULL;
             else {
+                const char * const sepr = gp_file_name_separator();
+                const int lsepr = strlen(sepr);
                 gp_getenv("PCLFONTSOURCE", path, &pathlen);     /* can't fail */
+
+                /* We want to ensure a trailing "/" is present */
+                if (gs_file_name_check_separator(path + (pathlen - (lsepr + 1)), lsepr, path + pathlen - 1) != 1) {
+                    strncat(path, gp_file_name_separator(), pathlen + 1);
+                }
+                code = gs_add_control_path(mem, gs_permit_file_reading, path);
+                if (code < 0) {
+                    gs_free_object(mem, path, "pjl_font_path");
+                    goto fail1;
+                }
                 pjlstate->environment_font_path = path;
             }
         } else                  /* environmet variable does not exist use pjl fontsource */
@@ -1805,7 +1885,7 @@ pjl_process_destroy(pjl_parser_state * pst)
         return;
 
     mem = pst->mem;
-    
+
     free_pjl_defaults(mem, &pst->defaults);
     free_pjl_environment(mem, &pst->envir);
     free_pjl_fontsource(mem, &pst->font_envir);
