@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -51,6 +51,7 @@
 #include "plchar.h"
 /* include the prototype for pixmap_high_level_pattern */
 #include "gsptype1.h"
+#include "gsgstate.h"
 
 /* Define whether to cache TrueType characters. */
 /* This would only be disabled for debugging. */
@@ -712,17 +713,6 @@ tt_find_table(gs_font_type42 * pfont, const char *tname, uint * plen)
     return 0;
 }
 
-#ifndef gs_gstate_DEFINED
-#  define gs_gstate_DEFINED
-typedef struct gs_gstate_s gs_gstate;
-#endif
-
-/* Opaque type for a path */
-#ifndef gx_path_DEFINED
-#  define gx_path_DEFINED
-typedef struct gx_path_s gx_path;
-#endif
-
 /*
  * Map a key through a cmap sub-table.  We export this so we can use
  * it someday for mapping between glyph vocabularies.  If the key is
@@ -907,9 +897,6 @@ pl_tt_encode_char(gs_font * pfont_generic, gs_char chr,
     pl_font_t *plfont = pfont->client_data;
     pl_font_glyph_t *pfg;
 
-    /* This is used by pl_decode_glyph in plfont.c */
-    plfont->last_char = chr;
-
     if (plfont->offsets.GC < 0)
         return glyph;           /* no substitute */
     pfg = pl_font_lookup_glyph(plfont, glyph);
