@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -22,11 +22,7 @@
 
 #include "gsiparam.h"
 #include "gsimage.h"
-
-#ifndef gs_gstate_DEFINED
-#  define gs_gstate_DEFINED
-typedef struct gs_gstate_s gs_gstate;
-#endif
+#include "gsgstate.h"
 
 /* Begin an image with parameters derived from a graphics state. */
 int pl_begin_image(gs_gstate * pgs, const gs_image_t * pim, void **pinfo);
