@@ -16,6 +16,7 @@
 
 /* plmain.c */
 /* Main program command-line interpreter for PCL interpreters */
+#include "ctype_.h"
 #include "string_.h"
 #include <stdlib.h> /* atof */
 #include "assert_.h"
@@ -140,7 +141,7 @@ static pl_interp_implementation_t
                               pl_main_instance_t * pmi, stream *s);
 
 /* Process the options on the command line. */
-static FILE *pl_main_arg_fopen(const char *fname, void *ignore_data);
+static gp_file *pl_main_arg_fopen(const char *fname, void *mem);
 
 /* Process the options on the command line, including making the
    initial device and setting its parameters.  */
@@ -209,8 +210,9 @@ pl_main_init_with_args(pl_main_instance_t *inst, int argc, char *argv[])
     gs_c_param_list_write(&inst->params, mem);
     gs_param_list_set_persistent_keys((gs_param_list *)&inst->params, false);
 
-    arg_init(&inst->args, (const char **)argv, argc, pl_main_arg_fopen, NULL,
-             inst->get_codepoint, mem);
+    if (arg_init(&inst->args, (const char **)argv, argc, pl_main_arg_fopen, mem,
+                 inst->get_codepoint, mem) < 0)
+        return gs_error_Fatal;
 
     /* Create PDL instances, etc */
     if (pl_main_languages_init(mem, inst) < 0) {
@@ -815,10 +817,11 @@ pl_top_create_device(pl_main_instance_t * pti, int index)
 }
 
 /* Process the options on the command line. */
-static FILE *
-pl_main_arg_fopen(const char *fname, void *ignore_data)
+static gp_file *
+pl_main_arg_fopen(const char *fname, void *data)
 {
-    return gp_fopen(fname, "r");
+    const gs_memory_t *mem = (const gs_memory_t *)data;
+    return gp_fopen(mem, fname, "r");
 }
 
 static void
@@ -836,7 +839,7 @@ set_debug_flags(const char *arg, char *flags)
  * number of arguments requested in the parameter arg_count.
  */
 static int
-parse_floats(gs_memory_t * mem, uint arg_count, char *arg, float *f)
+parse_floats(gs_memory_t * mem, uint arg_count, const char *arg, float *f)
 {
     int float_index = 0;
     char *tok, *l = NULL;
@@ -958,7 +961,7 @@ pl_main_post_args_init(pl_main_instance_t * pmi)
 }
 
 static int
-handle_dash_c(pl_main_instance_t *pmi, arg_list *pal, char **collected_commands, char **arg)
+handle_dash_c(pl_main_instance_t *pmi, arg_list *pal, char **collected_commands, const char **arg)
 {
     bool ats = pal->expand_ats;
     int code = 0;
@@ -971,6 +974,9 @@ handle_dash_c(pl_main_instance_t *pmi, arg_list *pal, char **collected_commands,
             ((*arg)[0] == '-' && !isdigit((unsigned char)(*arg)[1]))
             )
             break;
+        code = gs_lib_ctx_stash_sanitized_arg(pmi->memory->gs_lib_ctx, "?");
+        if (code < 0)
+            return code;
         arglen = strlen(*arg);
         if (*collected_commands == NULL) {
             *collected_commands = (char *)gs_alloc_bytes(pmi->memory, arglen+1,
@@ -1006,13 +1012,341 @@ problem_in_dash_c:
     return code;
 }
 
+static int
+set_param(pl_main_instance_t * pmi, const char *arg)
+{
+    /* We're setting a device parameter to a non-string value. */
+    const char *eqp = strchr(arg, '=');
+    const char *value;
+    int vi;
+    float vf;
+    bool bval = true;
+    char buffer[128];
+    pl_set_param_type spt_type = pl_spt_invalid;
+    const void *spt_val = NULL;
+    static const char const_true_string[] = "true";
+    int code = 0;
+    gs_c_param_list *params = &pmi->params;
+
+    if (eqp || (eqp = strchr(arg, '#')))
+        value = eqp + 1;
+    else {
+        /* -dDefaultBooleanIs_TRUE */
+        value = const_true_string;
+        eqp = arg + strlen(arg);
+    }
+
+    /* Arrange for a null terminated copy of the key name in buffer. */
+    if (eqp-arg >= sizeof(buffer)-1) {
+        dmprintf1(pmi->memory, "Command line key is too long: %s\n", arg);
+        return -1;
+    }
+    strncpy(buffer, arg, eqp - arg);
+    buffer[eqp - arg] = '\0';
+    if (value && value[0] == '/') {
+        /* We have a name! */
+        gs_param_string str;
+
+        code = check_for_special_str(pmi, arg, &str);
+        if (code <= 0)
+            return code;
+
+        param_string_from_transient_string(str, value + 1);
+        code = param_write_name((gs_param_list *) params,
+                                buffer, &str);
+        spt_type = pl_spt_name;
+        spt_val = value+1;
+    } else if (strchr(value, '#')) {
+        /* We have a non-decimal 'radix' number */
+        int base = 0;
+        const char *val = strchr(value, '#') + 1;
+        const char *v = value;
+        char c;
+
+        while ((c = *v++) >= '0' && c <= '9')
+            base = base*10 + (c - '0');
+        if (c != '#') {
+            dmprintf1(pmi->memory, "Malformed base value for radix. %s",
+                      value);
+            return -1;
+        }
+
+        if (base < 2 || base > 36) {
+            dmprintf1(pmi->memory, "Base out of range %s",
+                      value);
+            return -1;
+        }
+        vi = 0;
+        while (*val) {
+            if (*val >= '0' && *val < ('0'+(base<=10?base:10))) {
+                vi = vi * base + (*val - '0');
+            } else if (base > 10) {
+                if (*val >= 'A' && *val < 'A'+base-10) {
+                    vi = vi * base + (*val - 'A' + 10);
+                } else if (*val >= 'a' && *val < 'a'+base-10) {
+                    vi = vi * base + (*val - 'a' + 10);
+                } else {
+                    dmprintf1(pmi->memory,
+                              "Value out of range %s\n",
+                              val);
+                    return -1;
+                }
+            }
+            val++;
+        }
+        code = check_for_special_int(pmi, arg, vi);
+        if (code < 0) code = 0;
+        if (code <= 0)
+            return code;
+        code = param_write_int((gs_param_list *) params,
+                               buffer, &vi);
+        spt_type = pl_spt_int;
+        spt_val = &vi;
+    } else if ((!strchr(value, '.')) &&
+               (sscanf(value, "%d", &vi) == 1)) {
+        /* Here we have an int -- check for a scaling suffix */
+        char suffix = eqp[strlen(eqp) - 1];
+
+        switch (suffix) {
+            case 'k':
+            case 'K':
+                vi *= 1024;
+                break;
+            case 'm':
+            case 'M':
+                vi *= 1024 * 1024;
+                break;
+            case 'g':
+            case 'G':
+                /* caveat emptor: more than 2g will overflow */
+                /* and really should produce a 'real', so don't do this */
+                vi *= 1024 * 1024 * 1024;
+                break;
+            default:
+                break;  /* not a valid suffix or last char was digit */
+        }
+        code = check_for_special_int(pmi, arg, vi);
+        if (code < 0) code = 0;
+        if (code <= 0)
+            return code;
+        code = param_write_int((gs_param_list *) params,
+                               buffer, &vi);
+        spt_type = pl_spt_int;
+        spt_val = &vi;
+    } else if (sscanf(value, "%f", &vf) == 1) {
+        /* We have a float */
+        code = check_for_special_float(pmi, arg, vf);
+        if (code <= 0)
+            return code;
+        code = param_write_float((gs_param_list *) params,
+                                 buffer, &vf);
+        spt_type = pl_spt_float;
+        spt_val = &vf;
+    } else if (!strcmp(value, "null")) {
+        code = check_for_special_int(pmi, arg, (int)bval);
+        if (code < 0) code = 0;
+        if (code <= 0)
+            return code;
+        code = param_write_null((gs_param_list *) params,
+                                buffer);
+        spt_type = pl_spt_null;
+        spt_val = NULL;
+    } else if (!strcmp(value, "true")) {
+        /* bval = true; */
+        code = check_for_special_int(pmi, arg, (int)bval);
+        if (code < 0) code = 0;
+        if (code <= 0)
+            return code;
+        code = param_write_bool((gs_param_list *) params,
+                                buffer, &bval);
+        spt_type = pl_spt_bool;
+        spt_val = (void*)1;
+    } else if (!strcmp(value, "false")) {
+        bval = false;
+        code = check_for_special_int(pmi, arg, (int)bval);
+        if (code < 0) code = 0;
+        if (code <= 0)
+            return code;
+        code = param_write_bool((gs_param_list *) params,
+                                buffer, &bval);
+        spt_type = pl_spt_bool;
+        spt_val = NULL;
+    } else {
+        dmprintf(pmi->memory,
+                 "Usage for -d is -d<option>=[<integer>|<float>|null|true|false|name]\n");
+        arg = NULL;
+        return 0;
+    }
+    if (code < 0)
+        return code;
+    return pass_param_to_languages(pmi, spt_type, buffer, spt_val);
+}
+
+int
+pl_main_set_param(pl_main_instance_t * pmi, const char *arg)
+{
+    int code;
+    gs_c_param_list *params = &pmi->params;
+
+    gs_c_param_list_write_more(params);
+    code = set_param(pmi, arg);
+    if (code < 0) {
+        gs_c_param_list_release(params);
+        return code;
+    }
+
+    gs_c_param_list_read(params);
+    return gs_putdeviceparams(pmi->device, (gs_param_list *) params);
+}
+
+static int
+set_string_param(pl_main_instance_t * pmi, const char *arg)
+{
+    /* We're setting a device or user parameter to a string. */
+    char *eqp;
+    const char *value;
+    gs_param_string str;
+    int code = 0;
+    gs_c_param_list *params = &pmi->params;
+
+    eqp = strchr(arg, '=');
+    if (!(eqp || (eqp = strchr(arg, '#')))) {
+        return -1;
+    }
+    value = eqp + 1;
+    if (!strncmp(arg, "DEVICE", 6)) {
+        dmprintf(pmi->memory, "DEVICE can only be set on the command line!\n");
+        return -1;
+    } else if (!strncmp(arg, "DefaultGrayProfile",
+                        strlen("DefaultGrayProfile"))) {
+        dmprintf(pmi->memory, "DefaultGrayProfile can only be set on the command line!\n");
+        return -1;
+    } else if (!strncmp(arg, "DefaultRGBProfile",
+                        strlen("DefaultRGBProfile"))) {
+        dmprintf(pmi->memory, "DefaultRGBProfile can only be set on the command line!\n");
+        return -1;
+    } else if (!strncmp(arg, "DefaultCMYKProfile",
+                        strlen("DefaultCMYKProfile"))) {
+        dmprintf(pmi->memory, "DefaultCMYKProfile can only be set on the command line!\n");
+        return -1;
+        pmi->pdefault_cmyk_icc = arg_copy(value, pmi->memory);
+    } else if (!strncmp(arg, "ICCProfileDir", strlen("ICCProfileDir"))) {
+        dmprintf(pmi->memory, "ICCProfileDir can only be set on the command line!\n");
+        return -1;
+    } else {
+        char buffer[128];
+
+        if (eqp-arg >= sizeof(buffer)-1) {
+            dmprintf1(pmi->memory, "Command line key is too long: %s\n", arg);
+            return -1;
+        }
+        strncpy(buffer, arg, eqp - arg);
+        buffer[eqp - arg] = '\0';
+
+        param_string_from_transient_string(str, value);
+        code = param_write_string((gs_param_list *) params,
+                                  buffer, &str);
+        if (code < 0)
+            return code;
+        return pass_param_to_languages(pmi, pl_spt_string, buffer, value);
+    }
+    return code;
+}
+
+int
+pl_main_set_typed_param(pl_main_instance_t * pmi, pl_set_param_type type, const char *param, const void *value)
+{
+    int code = 0;
+    gs_c_param_list *params = &pmi->params;
+    gs_param_string str_value;
+    bool bval;
+
+    /* First set it in the device params */
+    gs_c_param_list_write_more(params);
+    switch (type)
+    {
+    case pl_spt_null:
+        code = param_write_null((gs_param_list *) params,
+                                param);
+        break;
+    case pl_spt_bool:
+        bval = (value != NULL);
+        code = param_write_bool((gs_param_list *) params,
+                                param, &bval);
+        break;
+    case pl_spt_int:
+        code = param_write_int((gs_param_list *) params,
+                               param, (int *)value);
+        break;
+    case pl_spt_float:
+        code = param_write_float((gs_param_list *) params,
+                                 param, (float *)value);
+        break;
+    case pl_spt_name:
+        param_string_from_transient_string(str_value, value);
+        code = param_write_name((gs_param_list *) params,
+                                param, &str_value);
+        break;
+    case pl_spt_string:
+        param_string_from_transient_string(str_value, value);
+        code = param_write_string((gs_param_list *) params,
+                                  param, &str_value);
+        break;
+    default:
+        code = gs_note_error(gs_error_rangecheck);
+    }
+    if (code < 0) {
+        gs_c_param_list_release(params);
+        return code;
+    }
+    gs_c_param_list_read(params);
+
+    /* Then send it to the languages */
+    return pass_param_to_languages(pmi, type, param, value);
+}
+
+int
+pl_main_set_string_param(pl_main_instance_t * pmi, const char *arg)
+{
+    int code;
+    gs_c_param_list *params = &pmi->params;
+
+    gs_c_param_list_write_more(params);
+    code = set_string_param(pmi, arg);
+    if (code < 0) {
+        gs_c_param_list_release(params);
+        return code;
+    }
+
+    gs_c_param_list_read(params);
+    return gs_putdeviceparams(pmi->device, (gs_param_list *) params);
+}
+
+static int
+do_arg_match(const char **arg, const char *match, size_t match_len)
+{
+    const char *s = *arg;
+    if (strncmp(s, match, match_len) != 0)
+        return 0;
+    s += match_len;
+    if (*s == '=')
+        *arg = ++s;
+    else if (*s != 0)
+        return 0;
+    else
+        *arg = NULL;
+    return 1;
+}
+
+#define arg_match(A, B) do_arg_match(A, B, sizeof(B)-1)
+
 static int
 pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         pl_interp_implementation_t * pjli)
 {
     int code = 0;
     bool help = false;
-    char *arg = NULL;
+    const char *arg = NULL;
     gs_c_param_list *params = &pmi->params;
     int device_index = -1;
     char *collected_commands = NULL;
@@ -1021,12 +1355,15 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
     /* By default, stdin_is_interactive = 1. This mirrors what stdin_init
      * would have done in the iodev one time initialisation. We aren't
      * getting our stdin via an iodev though, so we do it here. */
-   pmi->memory->gs_lib_ctx->core->stdin_is_interactive = 1;
+    pmi->memory->gs_lib_ctx->core->stdin_is_interactive = 1;
 
     gs_c_param_list_write_more(params);
     while (arg != NULL || (code = arg_next(pal, (const char **)&arg, pmi->memory)) > 0) {
         if (*arg != '-') /* Stop when we hit something that isn't an option */
             break;
+        code = gs_lib_ctx_stash_sanitized_arg(pmi->memory->gs_lib_ctx, arg);
+        if (code < 0)
+            return code;
         if (arg[1] == 0) {
             /* Stdin, not an option! */
             not_an_arg = 1;
@@ -1102,6 +1439,28 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
 #endif /* OMIT_SAVED_PAGES_TEST */
                     break;
                 }
+                /* Now handle the explicitly added paths to the file control lists */
+                else if (arg_match(&arg, "permit-file-read")) {
+                    code = gs_add_explicit_control_path(pmi->memory, arg, gs_permit_file_reading);
+                    if (code < 0) return code;
+                    break;
+                } else if (arg_match(&arg, "permit-file-write")) {
+                    code = gs_add_explicit_control_path(pmi->memory, arg, gs_permit_file_writing);
+                    if (code < 0) return code;
+                    break;
+                } else if (arg_match(&arg, "permit-file-control")) {
+                    code = gs_add_explicit_control_path(pmi->memory, arg, gs_permit_file_control);
+                    if (code < 0) return code;
+                    break;
+                } else if (arg_match(&arg, "permit-file-all")) {
+                    code = gs_add_explicit_control_path(pmi->memory, arg, gs_permit_file_reading);
+                    if (code < 0) return code;
+                    code = gs_add_explicit_control_path(pmi->memory, arg, gs_permit_file_writing);
+                    if (code < 0) return code;
+                    code = gs_add_explicit_control_path(pmi->memory, arg, gs_permit_file_control);
+                    if (code < 0) return code;
+                    break;
+                }
                 /* FALLTHROUGH */
             default:
                 dmprintf1(pmi->memory, "Unrecognized switch: %s\n", arg-2);
@@ -1114,179 +1473,15 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                 continue; /* Next arg has already been read, if there is one */
             case 'd':
             case 'D':
-                {
-                    /* We're setting a device parameter to a non-string value. */
-                    char *eqp = strchr(arg, '=');
-                    const char *value;
-                    int vi;
-                    float vf;
-                    bool bval = true;
-                    char buffer[128];
-                    pl_set_param_type spt_type = pl_spt_invalid;
-                    const void *spt_val = NULL;
-                    static const char const_true_string[] = "true";
-
-                    if (eqp || (eqp = strchr(arg, '#')))
-                        value = eqp + 1;
-                    else {
-                        /* -dDefaultBooleanIs_TRUE */
-                        value = const_true_string;
-                        eqp = arg + strlen(arg);
-                    }
-
-                    /* Arrange for a null terminated copy of the key name in buffer. */
-                    if (eqp-arg >= sizeof(buffer)-1) {
-                        dmprintf1(pmi->memory, "Command line key is too long: %s\n", arg);
-                        return -1;
-                    }
-                    strncpy(buffer, arg, eqp - arg);
-                    buffer[eqp - arg] = '\0';
-                    code = 0;
-                    if (value && value[0] == '/') {
-                        /* We have a name! */
-                        gs_param_string str;
-
-                        code = check_for_special_str(pmi, arg, &str);
-                        if (code <= 0)
-                            break;
-
-                        param_string_from_transient_string(str, value + 1);
-                        code = param_write_name((gs_param_list *) params,
-                                                buffer, &str);
-                        spt_type = pl_spt_name;
-                        spt_val = value+1;
-                    } else if (strchr(value, '#')) {
-                        /* We have a non-decimal 'radix' number */
-                        int base = 0;
-                        const char *val = strchr(value, '#') + 1;
-                        const char *v = value;
-                        char c;
-
-                        while ((c = *v++) >= '0' && c <= '9')
-                            base = base*10 + (c - '0');
-                        if (c != '#') {
-                            dmprintf1(pmi->memory, "Malformed base value for radix. %s",
-                                      value);
-                            return -1;
-                        }
-
-                        if (base < 2 || base > 36) {
-                            dmprintf1(pmi->memory, "Base out of range %s",
-                                      value);
-                            return -1;
-                        }
-                        vi = 0;
-                        while (*val) {
-                            if (*val >= '0' && *val < ('0'+(base<=10?base:10))) {
-                                vi = vi * base + (*val - '0');
-                            } else if (base > 10) {
-                                if (*val >= 'A' && *val < 'A'+base-10) {
-                                    vi = vi * base + (*val - 'A' + 10);
-                                } else if (*val >= 'a' && *val < 'a'+base-10) {
-                                    vi = vi * base + (*val - 'a' + 10);
-                                } else {
-                                    dmprintf1(pmi->memory,
-                                              "Value out of range %s\n",
-                                              val);
-                                    return -1;
-                                }
-                            }
-                            val++;
-                        }
-                        code = check_for_special_int(pmi, arg, vi);
-                        if (code < 0) code = 0;
-                        if (code <= 0)
-                            break;
-                        code = param_write_int((gs_param_list *) params,
-                                               buffer, &vi);
-                        spt_type = pl_spt_int;
-                        spt_val = &vi;
-                    } else if ((!strchr(value, '.')) &&
-                               (sscanf(value, "%d", &vi) == 1)) {
-                        /* Here we have an int -- check for a scaling suffix */
-                        char suffix = eqp[strlen(eqp) - 1];
-
-                        switch (suffix) {
-                            case 'k':
-                            case 'K':
-                                vi *= 1024;
-                                break;
-                            case 'm':
-                            case 'M':
-                                vi *= 1024 * 1024;
-                                break;
-                            case 'g':
-                            case 'G':
-                                /* caveat emptor: more than 2g will overflow */
-                                /* and really should produce a 'real', so don't do this */
-                                vi *= 1024 * 1024 * 1024;
-                                break;
-                            default:
-                                break;  /* not a valid suffix or last char was digit */
-                        }
-                        code = check_for_special_int(pmi, arg, vi);
-                        if (code < 0) code = 0;
-                        if (code <= 0)
-                            break;
-                        code = param_write_int((gs_param_list *) params,
-                                               buffer, &vi);
-                        spt_type = pl_spt_int;
-                        spt_val = &vi;
-                    } else if (sscanf(value, "%f", &vf) == 1) {
-                        /* We have a float */
-                        code = check_for_special_float(pmi, arg, vf);
-                        if (code <= 0)
-                            break;
-                        code = param_write_float((gs_param_list *) params,
-                                                  buffer, &vf);
-                        spt_type = pl_spt_float;
-                        spt_val = &vf;
-                    } else if (!strcmp(value, "null")) {
-                        code = check_for_special_int(pmi, arg, (int)bval);
-                        if (code < 0) code = 0;
-                        if (code <= 0)
-                            break;
-                        code = param_write_null((gs_param_list *) params,
-                                                buffer);
-                        spt_type = pl_spt_null;
-                        spt_val = NULL;
-                    } else if (!strcmp(value, "true")) {
-                        /* bval = true; */
-                        code = check_for_special_int(pmi, arg, (int)bval);
-                        if (code < 0) code = 0;
-                        if (code <= 0)
-                            break;
-                        code = param_write_bool((gs_param_list *) params,
-                                                buffer, &bval);
-                        spt_type = pl_spt_bool;
-                        spt_val = (void*)1;
-                    } else if (!strcmp(value, "false")) {
-                        bval = false;
-                        code = check_for_special_int(pmi, arg, (int)bval);
-                        if (code < 0) code = 0;
-                        if (code <= 0)
-                            break;
-                        code = param_write_bool((gs_param_list *) params,
-                                                buffer, &bval);
-                        spt_type = pl_spt_bool;
-                        spt_val = NULL;
-                    } else {
-                        dmprintf(pmi->memory,
-                                 "Usage for -d is -d<option>=[<integer>|<float>|null|true|false|name]\n");
-                        arg = NULL;
-                        continue;
-                    }
-                    if (code < 0)
-                        return code;
-                    code = pass_param_to_languages(pmi, spt_type, buffer, spt_val);
-                    if (code < 0)
-                        return code;
-                }
+                code = set_param(pmi, arg);
+                if (code < 0)
+                    return code;
+                break;
             case 'E':
                 if (*arg == 0)
                     gs_debug['#'] = 1;
                 else
-                    sscanf(arg, "%d", &pmi->error_report);
+                    (void)sscanf(arg, "%d", &pmi->error_report);
                 break;
             case 'f':
                 code = arg_next(pal, (const char **)&arg, pmi->memory);
@@ -1296,6 +1491,9 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                     dmprintf(pmi->memory, "-f must be followed by a filename\n");
                     continue;
                 }
+                code = gs_lib_ctx_stash_sanitized_arg(pmi->memory->gs_lib_ctx, "?");
+                if (code < 0)
+                    return code;
                 goto out;
             case 'g':
                 {
@@ -1347,6 +1545,9 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         code = arg_next(pal, (const char **)&path, pmi->memory);
                         if (code < 0)
                             return code;
+                        code = gs_lib_ctx_stash_sanitized_arg(pmi->memory->gs_lib_ctx, "?");
+                        if (code < 0)
+                            return code;
                     } else
                         path = arg;
                     if (path == NULL)
@@ -1447,6 +1648,9 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         code = arg_next(pal, (const char **)&adef, pmi->memory);
                         if (code < 0)
                             break;
+                        code = gs_lib_ctx_stash_sanitized_arg(pmi->memory->gs_lib_ctx, "?");
+                        if (code < 0)
+                            return code;
                     } else
                         adef = arg;
                     if (adef == NULL)
@@ -1454,6 +1658,9 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         code = gs_error_undefinedfilename;
                         break;
                     }
+                    code = gs_add_outputfile_control_path(pmi->memory, adef);
+                    if (code < 0)
+                        break;
                     param_string_from_transient_string(str, adef);
                     code =
                         param_write_string((gs_param_list *) params,
@@ -1531,10 +1738,9 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                 break;
             case 's':
             case 'S':
-                {               /* We're setting a device or user parameter to a string. */
+                {
                     char *eqp;
                     const char *value;
-                    gs_param_string str;
 
                     eqp = strchr(arg, '=');
                     if (!(eqp || (eqp = strchr(arg, '#')))) {
@@ -1551,48 +1757,30 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         device_index = get_device_index(pmi->memory, value);
                         if (device_index == -1)
                             return -1;
-                        /* check for icc settings */
-                    } else
-                        if (!strncmp
-                            (arg, "DefaultGrayProfile",
-                             strlen("DefaultGrayProfile"))) {
+                    } else if (!strncmp(arg, "DefaultGrayProfile",
+                                        strlen("DefaultGrayProfile"))) {
                         pmi->pdefault_gray_icc = arg_copy(value, pmi->memory);
-                    } else
-                        if (!strncmp
-                            (arg, "DefaultRGBProfile",
-                             strlen("DefaultRGBProfile"))) {
+                    } else if (!strncmp(arg, "DefaultRGBProfile",
+                                        strlen("DefaultRGBProfile"))) {
                         pmi->pdefault_rgb_icc = arg_copy(value, pmi->memory);
-                    } else
-                        if (!strncmp
-                            (arg, "DefaultCMYKProfile",
-                             strlen("DefaultCMYKProfile"))) {
+                    } else if (!strncmp(arg, "DefaultCMYKProfile",
+                                        strlen("DefaultCMYKProfile"))) {
                         pmi->pdefault_cmyk_icc = arg_copy(value, pmi->memory);
-                    } else
-                        if (!strncmp
-                            (arg, "ICCProfileDir", strlen("ICCProfileDir"))) {
+                    } else if (!strncmp(arg, "ICCProfileDir", strlen("ICCProfileDir"))) {
                         pmi->piccdir = arg_copy(value, pmi->memory);
-                    } else {
-                        char buffer[128];
-
-                        if (eqp-arg >= sizeof(buffer)-1) {
-                            dmprintf1(pmi->memory, "Command line key is too long: %s\n", arg);
-                            return -1;
-                        }
-                        strncpy(buffer, arg, eqp - arg);
-                        buffer[eqp - arg] = '\0';
-
-                        param_string_from_transient_string(str, value);
-                        code =
-                            param_write_string((gs_param_list *) params,
-                                               buffer, &str);
+                    } else if (!strncmp(arg, "OutputFile", 10) && strlen(eqp) > 0) {
+                        code = gs_add_outputfile_control_path(pmi->memory, eqp+1);
+                        if (code < 0) return code;
+                        code = set_string_param(pmi, arg);
                         if (code < 0)
                             return code;
-                        code = pass_param_to_languages(pmi, pl_spt_string, buffer, value);
+                    } else {
+                        code = set_string_param(pmi, arg);
                         if (code < 0)
                             return code;
                     }
+                    break;
                 }
-                break;
             case 'Z':
                 set_debug_flags(arg, gs_debug);
                 break;
@@ -1640,7 +1828,17 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
          * a '-', they'll do "-f -blah". The - of the "-blah" must not be accepted
          * by the arg processing in the loop below. We use 'not_an_arg' to handle
          * this. */
-        do {
+        while (1) {
+            if (arg == NULL) {
+                code = arg_next(pal, (const char **)&arg, pmi->memory);
+                if (code < 0)
+                    break;
+                if (arg == NULL)
+                    break;
+                code = gs_lib_ctx_stash_sanitized_arg(pmi->memory->gs_lib_ctx, arg);
+                if (code < 0)
+                    return code;
+            }
             if (!not_an_arg && arg[0] == '-' && arg[1] == 'c') {
                 code = handle_dash_c(pmi, pal, &collected_commands, &arg);
                 if (code < 0)
@@ -1655,10 +1853,17 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                     dmprintf(pmi->memory, "-f must be followed by a filename\n");
                     continue;
                 }
+                code = gs_lib_ctx_stash_sanitized_arg(pmi->memory->gs_lib_ctx, "?");
+                if (code < 0)
+                    return code;
                 not_an_arg = 1;
                 continue;
             } else {
+                code = gs_add_control_path(pmi->memory, gs_permit_file_reading, arg);
+                if (code < 0)
+                    break;
                 code = pl_main_run_file_utf8(pmi, collected_commands, arg);
+                (void)gs_remove_control_path(pmi->memory, gs_permit_file_reading, arg);
                 if (code == gs_error_undefinedfilename)
                     errprintf(pmi->memory, "Failed to open file '%s'\n", arg);
                 gs_free_object(pmi->memory, collected_commands, "-c buffer");
@@ -1668,7 +1873,7 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
             }
             arg = NULL;
             not_an_arg = 0;
-        } while (arg != NULL || (code = arg_next(pal, (const char **)&arg, pmi->memory)) > 0);
+        }
     }
 
     if (code == 0 && collected_commands != NULL) {
