@@ -71,6 +71,7 @@ typedef gs_memory_t * (*pl_interp_proc_get_device_memory_t) (pl_interp_implement
 
 /*
  * Pass a parameter/value to a language.
+ * Note: Keep this in sync with gs_set_param_type from iapi.h.
  */
 typedef enum {
     pl_spt_invalid = -1,
@@ -90,6 +91,7 @@ typedef int (*pl_interp_proc_set_param_t) (pl_interp_implementation_t *,
 /*
  * Add a path to a language's search path.
  */
+int pl_add_path(pl_interp_implementation_t *impl, const char *path);
 typedef int (*pl_interp_proc_add_path_t) (pl_interp_implementation_t *,
                                           const char *);
 
