@@ -114,7 +114,7 @@ static void print_help_trailer(const gs_main_instance *);
 /* ------ Main program ------ */
 
 /* Process the command line with a given instance. */
-static FILE *
+static gp_file *
 gs_main_arg_fopen(const char *fname, void *vminst)
 {
     gs_main_set_lib_paths((gs_main_instance *) vminst);
@@ -138,10 +138,13 @@ gs_main_init_with_args01(gs_main_instance * minst, int argc, char *argv[])
     int code;
     int have_dumped_args = 0;
 
-    arg_init(&args, (const char **)argv, argc,
-             gs_main_arg_fopen, (void *)minst,
-             minst->get_codepoint,
-             minst->heap);
+    /* Now we actually process them */
+    code = arg_init(&args, (const char **)argv, argc,
+                    gs_main_arg_fopen, (void *)minst,
+                    minst->get_codepoint,
+                    minst->heap);
+    if (code < 0)
+        return code;
     code = gs_main_init0(minst, 0, 0, 0, GS_MAX_LIB_DIRS);
     if (code < 0)
         return code;
@@ -213,6 +216,9 @@ gs_main_init_with_args01(gs_main_instance * minst, int argc, char *argv[])
         }
     }
     while ((code = arg_next(&args, (const char **)&arg, minst->heap)) > 0) {
+        code = gs_lib_ctx_stash_sanitized_arg(minst->heap->gs_lib_ctx, arg);
+        if (code < 0)
+            return code;
         switch (*arg) {
             case '-':
                 code = swproc(minst, arg, &args);
@@ -309,6 +315,24 @@ gs_main_run_start(gs_main_instance * minst)
     return run_string(minst, "systemdict /start get exec", runFlush, minst->user_errors, NULL, NULL);
 }
 
+static int
+do_arg_match(const char **arg, const char *match, size_t match_len)
+{
+    const char *s = *arg;
+    if (strncmp(s, match, match_len) != 0)
+        return 0;
+    s += match_len;
+    if (*s == '=')
+        *arg = ++s;
+    else if (*s != 0)
+        return 0;
+    else
+        *arg = NULL;
+    return 1;
+}
+
+#define arg_match(A, B) do_arg_match(A, B, sizeof(B)-1)
+
 /* Process switches.  Return 0 if processed, 1 for unknown switch, */
 /* <0 if error. */
 static int
@@ -428,6 +452,27 @@ run_stdin:
             } else if (strncmp(arg, "saved-pages-test", 16) == 0) {
                 minst->saved_pages_test_mode = true;
                 break;
+            /* Now handle the explicitly added paths to the file control lists */
+            } else if (arg_match(&arg, "permit-file-read")) {
+                code = gs_add_explicit_control_path(minst->heap, arg, gs_permit_file_reading);
+                if (code < 0) return code;
+                break;
+            } else if (arg_match(&arg, "permit-file-write")) {
+                code = gs_add_explicit_control_path(minst->heap, arg, gs_permit_file_writing);
+                if (code < 0) return code;
+                break;
+            } else if (arg_match(&arg, "permit-file-control")) {
+                code = gs_add_explicit_control_path(minst->heap, arg, gs_permit_file_control);
+                if (code < 0) return code;
+                break;
+            } else if (arg_match(&arg, "permit-file-all")) {
+                code = gs_add_explicit_control_path(minst->heap, arg, gs_permit_file_reading);
+                if (code < 0) return code;
+                code = gs_add_explicit_control_path(minst->heap, arg, gs_permit_file_writing);
+                if (code < 0) return code;
+                code = gs_add_explicit_control_path(minst->heap, arg, gs_permit_file_control);
+                if (code < 0) return code;
+                break;
             }
             /* FALLTHROUGH */
         case '+':
@@ -438,6 +483,7 @@ run_stdin:
                 char *psarg;
 
                 code = arg_next(pal, (const char **)&psarg, minst->heap);
+                /* Don't stash the @ file name */
 
                 if (code < 0)
                     return gs_error_Fatal;
@@ -455,6 +501,9 @@ run_stdin:
                     code = run_string(minst, "userdict/ARGUMENTS[", 0, minst->user_errors, NULL, NULL);
                 if (code >= 0)
                     while ((code = arg_next(pal, (const char **)&arg, minst->heap)) > 0) {
+                        code = gs_lib_ctx_stash_sanitized_arg(minst->heap->gs_lib_ctx, arg);
+                        if (code < 0)
+                            break;
                         code = runarg(minst, "", arg, "", runInit, minst->user_errors, NULL, NULL);
                         if (code < 0)
                             break;
@@ -467,19 +516,6 @@ run_stdin:
 
                 return code;
             }
-        case 'A':               /* trace allocator */
-            switch (*arg) {
-                case 0:
-                    gs_alloc_debug = 1;
-                    break;
-                case '-':
-                    gs_alloc_debug = 0;
-                    break;
-                default:
-                    puts(minst->heap, "-A may only be followed by -");
-                    return gs_error_Fatal;
-            }
-            break;
         case 'B':               /* set run_string buffer size */
             if (*arg == '-')
                 minst->run_buffer_size = 0;
@@ -510,6 +546,9 @@ run_stdin:
                         (arg[0] == '-' && !isdigit((unsigned char)arg[1]))
                         )
                         break;
+                    code = gs_lib_ctx_stash_sanitized_arg(minst->heap->gs_lib_ctx, "?");
+                    if (code < 0)
+                        return code;
                     code = runarg(minst, "", arg, ".runstring", 0, minst->user_errors, NULL, NULL);
                     if (code < 0)
                         return code;
@@ -525,19 +564,6 @@ run_stdin:
                 pal->expand_ats = ats;
                 break;
             }
-        case 'E':               /* log errors */
-            switch (*arg) {
-                case 0:
-                    gs_log_errors = 1;
-                    break;
-                case '-':
-                    gs_log_errors = 0;
-                    break;
-                default:
-                    puts(minst->heap, "-E may only be followed by -");
-                    return gs_error_Fatal;
-            }
-            break;
         case 'f':               /* run file of arbitrary name */
             if (*arg != 0) {
                 code = argproc(minst, arg);
@@ -628,6 +654,9 @@ run_stdin:
                     code = arg_next(pal, (const char **)&path, minst->heap);
                     if (code < 0)
                         return code;
+                    code = gs_lib_ctx_stash_sanitized_arg(minst->heap->gs_lib_ctx, "?");
+                    if (code < 0)
+                        return code;
                 } else
                     path = arg;
                 if (path == NULL)
@@ -640,7 +669,7 @@ run_stdin:
                 long msize = 0;
                 gs_malloc_memory_t *rawheap = gs_malloc_wrapped_contents(minst->heap);
 
-                sscanf((const char *)arg, "%ld", &msize);
+                (void)sscanf((const char *)arg, "%ld", &msize);
                 if (msize <= 0 || msize > max_long >> 10) {
                     outprintf(minst->heap, "-K<numK> must have 1 <= numK <= %ld\n",
                               max_long >> 10);
@@ -653,7 +682,7 @@ run_stdin:
             {
                 unsigned msize = 0;
 
-                sscanf((const char *)arg, "%u", &msize);
+                (void)sscanf((const char *)arg, "%u", &msize);
                 if (msize <= 0 || msize >= (max_uint >> 10)) {
                     outprintf(minst->heap, "-M must be between 1 and %d\n", (int)((max_uint >> 10) - 1));
                     return gs_error_Fatal;
@@ -665,7 +694,7 @@ run_stdin:
             {
                 unsigned nsize = 0;
 
-                sscanf((const char *)arg, "%d", &nsize);
+                (void)sscanf((const char *)arg, "%d", &nsize);
                 if (nsize < 2 || nsize > (max_uint >> 10)) {
                     outprintf(minst->heap, "-N must be between 2 and %d\n", (int)(max_uint >> 10));
                     return gs_error_Fatal;
@@ -693,10 +722,19 @@ run_stdin:
                         return code;
                     if (code == 0)
                         return gs_error_undefinedfilename;
+                    code = gs_lib_ctx_stash_sanitized_arg(minst->heap->gs_lib_ctx, "?");
+                    if (code < 0)
+                        return code;
                 } else
                     adef = arg;
                 if ((code = gs_main_init1(minst)) < 0)
                     return code;
+
+                if (strlen(adef) > 0) {
+                    code = gs_add_outputfile_control_path(minst->heap, adef);
+                    if (code < 0) return code;
+                }
+
                 ialloc_set_space(idmemory, avm_system);
                 len = strlen(adef);
                 str = ialloc_string(len, "-o");
@@ -781,6 +819,12 @@ run_stdin:
                     uint space = icurrent_space;
 
                     *eqp++ = 0;
+
+                    if (strlen(adef) == 10 && strncmp(adef, "OutputFile", 10) == 0 && strlen(eqp) > 0) {
+                        code = gs_add_outputfile_control_path(minst->heap, eqp);
+                        if (code < 0) return code;
+                    }
+
                     ialloc_set_space(idmemory, avm_system);
                     if (isd) {
                         int num, i;
@@ -948,22 +992,31 @@ esc_strcat(char *dest, const char *src)
 static int
 argproc(gs_main_instance * minst, const char *arg)
 {
-    int code = gs_main_init1(minst);            /* need i_ctx_p to proceed */
+    int code1, code = gs_main_init1(minst);            /* need i_ctx_p to proceed */
 
     if (code < 0)
         return code;
+
+    code = gs_add_control_path(minst->heap, gs_permit_file_reading, arg);
+    if (code < 0) return code;
+
     if (minst->run_buffer_size) {
         /* Run file with run_string. */
-        return run_buffered(minst, arg);
+        code = run_buffered(minst, arg);
     } else {
         /* Run file directly in the normal way. */
-        return runarg(minst, "", arg, ".runfile", runInit | runFlush, minst->user_errors, NULL, NULL);
+        code = runarg(minst, "", arg, ".runfile", runInit | runFlush, minst->user_errors, NULL, NULL);
     }
+
+    code1 = gs_remove_control_path(minst->heap, gs_permit_file_reading, arg);
+    if (code >= 0 && code1 < 0) code = code1;
+
+    return code;
 }
 static int
 run_buffered(gs_main_instance * minst, const char *arg)
 {
-    FILE *in = gp_fopen(arg, gp_fmode_rb);
+    gp_file *in = gp_fopen(minst->heap, arg, gp_fmode_rb);
     int exit_code;
     ref error_object;
     int code;
@@ -974,7 +1027,7 @@ run_buffered(gs_main_instance * minst, const char *arg)
     }
     code = gs_main_init2(minst);
     if (code < 0) {
-        fclose(in);
+        gp_fclose(in);
         return code;
     }
     code = gs_main_run_string_begin(minst, minst->user_errors,
@@ -984,7 +1037,7 @@ run_buffered(gs_main_instance * minst, const char *arg)
         int count;
 
         code = gs_error_NeedInput;
-        while ((count = fread(buf, 1, minst->run_buffer_size, in)) > 0) {
+        while ((count = gp_fread(buf, 1, minst->run_buffer_size, in)) > 0) {
             code = gs_main_run_string_continue(minst, buf, count,
                                                minst->user_errors,
                                                &exit_code, &error_object);
@@ -996,7 +1049,7 @@ run_buffered(gs_main_instance * minst, const char *arg)
                                           &exit_code, &error_object);
         }
     }
-    fclose(in);
+    gp_fclose(in);
     zflush(minst->i_ctx_p);
     zflushpage(minst->i_ctx_p);
     return run_finish(minst, code, exit_code, &error_object);
@@ -1113,20 +1166,21 @@ try_stdout_redirect(gs_main_instance * minst,
          * or stderr, close it
          */
         if (core->fstdout2
-            && (core->fstdout2 != core->fstdout)
-            && (core->fstdout2 != core->fstderr)) {
-            fclose(core->fstdout2);
-            core->fstdout2 = (FILE *)NULL;
+            && (gp_get_file(core->fstdout2) != core->fstdout)
+            && (gp_get_file(core->fstdout2) != core->fstderr)) {
+            gp_fclose(core->fstdout2);
+            core->fstdout2 = NULL;
         }
         /* If stdout is being redirected, set minst->fstdout2 */
         if ( (filename != 0) && strlen(filename) &&
             strcmp(filename, "-") && strcmp(filename, "%stdout") ) {
             if (strcmp(filename, "%stderr") == 0) {
                 core->stdout_to_stderr = 1;
+            } else {
+                core->fstdout2 = gp_fopen(minst->heap, filename, "w");
+                if (core->fstdout2 == NULL)
+                    return_error(gs_error_invalidfileaccess);
             }
-            else if ((core->fstdout2 =
-                      gp_fopen(filename, "w")) == (FILE *)NULL)
-                return_error(gs_error_invalidfileaccess);
             core->stdout_is_redirected = 1;
         }
         return 0;
