@@ -47,7 +47,7 @@ struct xps_interp_instance_s
     gs_memory_t *memory;                /* memory allocator to use */
 
     xps_context_t *ctx;
-    FILE *scratch_file;
+    gp_file *scratch_file;
     char scratch_name[gp_file_name_sizeof];
 };
 
@@ -292,7 +292,7 @@ xps_impl_process(pl_interp_implementation_t *impl, stream_cursor_read *cursor)
     }
 
     avail = cursor->limit - cursor->ptr;
-    n = fwrite(cursor->ptr + 1, 1, avail, instance->scratch_file);
+    n = gp_fwrite(cursor->ptr + 1, 1, avail, instance->scratch_file);
     if (n != avail)
     {
         gs_catch(gs_error_invalidfileaccess, "cannot write to scratch file");
@@ -331,7 +331,7 @@ xps_impl_process_eof(pl_interp_implementation_t *impl)
     if (instance->scratch_file)
     {
         if_debug0m('|', ctx->memory, "xps: executing scratch file\n");
-        fclose(instance->scratch_file);
+        gp_fclose(instance->scratch_file);
         instance->scratch_file = NULL;
         code = xps_process_file(ctx, instance->scratch_name);
         unlink(instance->scratch_name);
@@ -366,6 +366,12 @@ static void xps_free_font_func(xps_context_t *ctx, void *ptr)
     xps_free_font(ctx, ptr);
 }
 
+static void xps_free_hashed_colorspace(xps_context_t *ctx, void *ptr)
+{
+    gs_color_space *cs = (gs_color_space *)ptr;
+    rc_decrement(cs, "xps_free_hashed_colorspace");
+}
+
 /* Wrap up interp instance after a "job" */
 static int
 xps_impl_dnit_job(pl_interp_implementation_t *impl)
@@ -386,7 +392,7 @@ xps_impl_dnit_job(pl_interp_implementation_t *impl)
 
     /* TODO: free resources too */
     xps_hash_free(ctx, ctx->font_table, xps_free_key_func, xps_free_font_func);
-    xps_hash_free(ctx, ctx->colorspace_table, xps_free_key_func, NULL);
+    xps_hash_free(ctx, ctx->colorspace_table, xps_free_key_func, xps_free_hashed_colorspace);
 
     xps_free_fixed_pages(ctx);
     xps_free_fixed_documents(ctx);
@@ -409,8 +415,10 @@ xps_impl_deallocate_interp_instance(pl_interp_implementation_t *impl)
     rc_decrement_cs(ctx->srgb, "xps_impl_deallocate_interp_instance");
     rc_decrement_cs(ctx->scrgb, "xps_impl_deallocate_interp_instance");
 
+    gx_pattern_cache_free(ctx->pgs->pattern_cache);
     gs_gstate_free(ctx->pgs);
 
+    gs_free_object(mem, ctx->start_part, "xps_impl_deallocate_interp_instance");
     gs_free_object(mem, ctx->fontdir, "xps_impl_deallocate_interp_instance");
     gs_free_object(mem, ctx, "xps_impl_deallocate_interp_instance");
     gs_free_object(mem, instance, "xps_impl_deallocate_interp_instance");
